"""Simplify dataset (minimize the dataset size).

Init:
pick up init data from dataset randomly

Iter:
00: train models (same as generator)
01: calculate model deviations of the rest dataset, pick up data with proper model deviaiton 
02: fp (optional, if the original dataset do not have fp data, same as generator)
"""
import logging
import warnings
import queue
import os
import json
import argparse
import pickle
import glob
import fnmatch
import dpdata
import numpy as np

from dpgen import dlog
from dpgen import SHORT_CMD
from dpgen.util import sepline, expand_sys_str
from distutils.version import LooseVersion
from dpgen.dispatcher.Dispatcher import Dispatcher, _split_tasks, make_dispatcher, make_submission
from dpgen.generator.run import make_train, run_train, post_train, run_fp, post_fp, fp_name, model_devi_name, train_name, train_task_fmt, sys_link_fp_vasp_pp, make_fp_vasp_incar, make_fp_vasp_kp, make_fp_vasp_cp_cvasp, data_system_fmt, model_devi_task_fmt, fp_task_fmt
# TODO: maybe the following functions can be moved to dpgen.util
from dpgen.generator.lib.utils import log_iter, make_iter_name, create_path, record_iter
from dpgen.generator.lib.gaussian import make_gaussian_input
from dpgen.remote.decide_machine import  convert_mdata


picked_data_name = "data.picked"
rest_data_name = "data.rest"
accurate_data_name = "data.accurate"
detail_file_name_prefix = "details"
sys_name_fmt = 'sys.' + data_system_fmt
sys_name_pattern = 'sys.[0-9]*[0-9]'


def get_system_cls(jdata):
    if jdata.get("labeled", False):
        return dpdata.LabeledSystem
    return dpdata.System


def get_multi_system(path, jdata):
    system = get_system_cls(jdata)
    system_paths = expand_sys_str(path)
    systems = dpdata.MultiSystems(
        *[system(s, fmt='deepmd/npy') for s in system_paths])
    return systems


def init_model(iter_index, jdata, mdata):
    training_init_model = jdata.get('training_init_model', False)
    if not training_init_model:
        return
    iter0_models = []
    training_iter0_model = jdata.get('training_iter0_model_path', [])
    if type(training_iter0_model) == str:
        training_iter0_model = [training_iter0_model]
    for ii in training_iter0_model:            
        model_is = glob.glob(ii)
        model_is.sort()
        iter0_models += [os.path.abspath(ii) for ii in model_is]
    numb_models = jdata['numb_models']
    assert(numb_models == len(iter0_models)), "training_iter0_model_path should be provided, and the number of models should be equal to %d" % numb_models
    work_path = os.path.join(make_iter_name(iter_index), train_name)
    create_path(work_path)
    cwd = os.getcwd()
    for ii in range(len(iter0_models)):
        train_path = os.path.join(work_path, train_task_fmt % ii)
        create_path(train_path)
        os.chdir(train_path)
        ckpt_files = glob.glob(os.path.join(iter0_models[ii], 'model.ckpt*'))
        for jj in ckpt_files:
            os.symlink(jj, os.path.basename(jj))
        os.chdir(cwd)


def init_pick(iter_index, jdata, mdata):
    """pick up init data from dataset randomly"""
    pick_data = jdata['pick_data']
    init_pick_number = jdata['init_pick_number']
    # use MultiSystems with System
    # TODO: support System and LabeledSystem
    # TODO: support other format
    systems = get_multi_system(pick_data, jdata)
    # label the system
    labels = []
    items = systems.systems.items()
    for key, system in items:
        labels.extend([(key, j) for j in range(len(system))])

    # random pick
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, model_devi_name)
    create_path(work_path)
    idx = np.arange(len(labels))
    np.random.shuffle(idx)
    pick_idx = idx[:init_pick_number]
    rest_idx = idx[init_pick_number:]

    # dump the init data
    sys_data_path = os.path.join(work_path, picked_data_name)
    _init_dump_selected_frames(systems, labels, pick_idx, sys_data_path, jdata)

    # dump the rest data
    sys_data_path = os.path.join(work_path, rest_data_name)
    _init_dump_selected_frames(systems, labels, rest_idx, sys_data_path, jdata)


def _init_dump_selected_frames(systems, labels, selc_idx, sys_data_path, jdata):
    selc_systems = dpdata.MultiSystems()
    for j in selc_idx:
        sys_name, sys_id = labels[j]
        selc_systems.append(systems[sys_name][sys_id])
    selc_systems.to_deepmd_raw(sys_data_path)
    selc_systems.to_deepmd_npy(sys_data_path, set_size=selc_idx.size)


def make_model_devi(iter_index, jdata, mdata):
    """calculate the model deviation of the rest idx"""
    pick_data = jdata['pick_data']
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, model_devi_name)
    create_path(work_path)
    # link the model
    train_path = os.path.join(iter_name, train_name)
    train_path = os.path.abspath(train_path)
    models = glob.glob(os.path.join(train_path, "graph*pb"))
    for mm in models:
        model_name = os.path.basename(mm)
        os.symlink(mm, os.path.join(work_path, model_name))
    # link the last rest data
    last_iter_name = make_iter_name(iter_index-1)
    rest_data_path = os.path.join(last_iter_name, model_devi_name, rest_data_name)
    if not os.path.exists(rest_data_path):
        return False
    os.symlink(os.path.abspath(rest_data_path), os.path.join(work_path, rest_data_name + ".old"))
    return True


def run_model_devi(iter_index, jdata, mdata):
    """submit dp test tasks"""
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, model_devi_name)
    # generate command
    commands = []
    run_tasks = ["."]
    # get models
    models = glob.glob(os.path.join(work_path, "graph*pb"))
    model_names = [os.path.basename(ii) for ii in models]
    task_model_list = []
    for ii in model_names:
        task_model_list.append(os.path.join('.', ii))
    # models
    commands = []
    detail_file_name = detail_file_name_prefix
    command = "{dp} model-devi -m {model} -s {system} -o {detail_file}".format(
        dp=mdata.get('model_devi_command', 'dp'),
        model=" ".join(task_model_list),
        system=rest_data_name + ".old",
        detail_file=detail_file_name,
    )
    commands = [command]
    # submit
    model_devi_group_size = mdata.get('model_devi_group_size', 1)

    forward_files = [rest_data_name + ".old"]
    backward_files = [detail_file_name]

    api_version = mdata.get('api_version', '0.9')
    if LooseVersion(api_version) < LooseVersion('1.0'):
        warnings.warn(f"the dpdispatcher will be updated to new version."
            f"And the interface may be changed. Please check the documents for more details")
        dispatcher = make_dispatcher(mdata['model_devi_machine'], mdata['model_devi_resources'], work_path, run_tasks, model_devi_group_size)
        dispatcher.run_jobs(mdata['model_devi_resources'],
                        commands,
                        work_path,
                        run_tasks,
                        model_devi_group_size,
                        model_names,
                        forward_files,
                        backward_files,
                        outlog = 'model_devi.log',
                        errlog = 'model_devi.log')

    elif LooseVersion(api_version) >= LooseVersion('1.0'):
        submission = make_submission(
            mdata['model_devi_machine'],
            mdata['model_devi_resources'],
            commands=commands,
            work_path=work_path,
            run_tasks=run_tasks,
            group_size=model_devi_group_size,
            forward_common_files=model_names,
            forward_files=forward_files,
            backward_files=backward_files,
            outlog = 'model_devi.log',
            errlog = 'model_devi.log')
        submission.run_submission()


def post_model_devi(iter_index, jdata, mdata):
    """calculate the model deviation"""
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, model_devi_name)

    f_trust_lo = jdata['model_devi_f_trust_lo']
    f_trust_hi = jdata['model_devi_f_trust_hi']

    sys_accurate = dpdata.MultiSystems()
    sys_candinate = dpdata.MultiSystems()
    sys_failed = dpdata.MultiSystems()
    
    labeled = jdata.get("labeled", False)
    sys_entire = dpdata.MultiSystems().from_deepmd_npy(os.path.join(work_path, rest_data_name + ".old"), labeled=labeled)

    detail_file_name = detail_file_name_prefix
    with open(os.path.join(work_path, detail_file_name)) as f:
        for line in f:
            if line.startswith("# data.rest.old"):
                name = (line.split()[1]).split("/")[-1]
            elif line.startswith("#"):
                pass
            else:
                idx = int(line.split()[0])
                f_devi = float(line.split()[4])
                subsys = sys_entire[name][idx]
                if f_trust_lo <= f_devi < f_trust_hi:
                    sys_candinate.append(subsys)
                elif f_devi >= f_trust_hi:
                    sys_failed.append(subsys)
                elif f_devi < f_trust_lo:
                    sys_accurate.append(subsys)
                else:
                    raise RuntimeError('reach a place that should NOT be reached...')

    counter = {"candidate": sys_candinate.get_nframes(), "accurate": sys_accurate.get_nframes(), "failed": sys_failed.get_nframes()}
    fp_sum = sum(counter.values())
    for cc_key, cc_value in counter.items():
        dlog.info("{0:9s} : {1:6d} in {2:6d} {3:6.2f} %".format(cc_key, cc_value, fp_sum, cc_value/fp_sum*100))
    
    if counter['candidate'] == 0 and counter['failed'] > 0:
        raise RuntimeError('no candidate but still have failed cases, stop. You may want to refine the training or to increase the trust level hi')

    # label the candidate system
    labels = []
    items = sys_candinate.systems.items()

    for key, system in items:
        labels.extend([(key, j) for j in range(len(system))])
    # candinate: pick up randomly
    iter_pick_number = jdata['iter_pick_number']
    idx = np.arange(counter['candidate'])
    assert(len(idx) == len(labels))
    np.random.shuffle(idx)
    pick_idx = idx[:iter_pick_number]
    rest_idx = idx[iter_pick_number:]
    if(counter['candidate'] == 0) : 
        dlog.info("no candidate")
    else :
        dlog.info("total candidate {0:6d}   picked {1:6d} ({2:6.2f} %) rest {3:6d} ({4:6.2f} % )".format\
              (counter['candidate'], len(pick_idx), float(len(pick_idx))/counter['candidate']*100., len(rest_idx), float(len(rest_idx))/counter['candidate']*100.))

    # dump the picked candinate data
    picked_systems = dpdata.MultiSystems()
    for j in pick_idx:
        sys_name, sys_id = labels[j]
        picked_systems.append(sys_candinate[sys_name][sys_id])
    sys_data_path = os.path.join(work_path, picked_data_name)
    picked_systems.to_deepmd_raw(sys_data_path)
    picked_systems.to_deepmd_npy(sys_data_path, set_size=iter_pick_number)


    # dump the rest data (not picked candinate data and failed data)
    rest_systems = dpdata.MultiSystems()
    for j in rest_idx:
        sys_name, sys_id = labels[j]
        rest_systems.append(sys_candinate[sys_name][sys_id])
    rest_systems += sys_failed
    sys_data_path = os.path.join(work_path, rest_data_name)
    rest_systems.to_deepmd_raw(sys_data_path)
    if rest_idx.size:
        rest_systems.to_deepmd_npy(sys_data_path, set_size=rest_idx.size)


    # dump the accurate data -- to another directory
    sys_data_path = os.path.join(work_path, accurate_data_name)
    sys_accurate.to_deepmd_raw(sys_data_path)
    sys_accurate.to_deepmd_npy(sys_data_path, set_size=sys_accurate.get_nframes())


def make_fp_labeled(iter_index, jdata):    
    dlog.info("already labeled, skip make_fp and link data directly")
    pick_data = jdata['pick_data']
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, fp_name)
    create_path(work_path)
    picked_data_path = os.path.join(iter_name, model_devi_name, picked_data_name)
    os.symlink(os.path.abspath(picked_data_path), os.path.abspath(
        os.path.join(work_path, "task." + data_system_fmt % 0)))
    os.symlink(os.path.abspath(picked_data_path), os.path.abspath(
        os.path.join(work_path, "data." + data_system_fmt % 0)))


def make_fp_configs(iter_index, jdata):
    pick_data = jdata['pick_data']
    iter_name = make_iter_name(iter_index)
    work_path = os.path.join(iter_name, fp_name)
    create_path(work_path)
    picked_data_path = os.path.join(iter_name, model_devi_name, picked_data_name)
    systems = get_multi_system(picked_data_path, jdata)
    jj = 0
    for system in systems:
        for subsys in system:
            task_name = "task." + fp_task_fmt % (0, jj)
            task_path = os.path.join(work_path, task_name)
            create_path(task_path)
            subsys.to('vasp/poscar', os.path.join(task_path, 'POSCAR'))
            jj += 1


def make_fp_gaussian(iter_index, jdata):
    work_path = os.path.join(make_iter_name(iter_index), fp_name)
    fp_tasks = glob.glob(os.path.join(work_path, 'task.*'))
    cwd = os.getcwd()
    if 'user_fp_params' in jdata.keys() :
        fp_params = jdata['user_fp_params']
    else:
        fp_params = jdata['fp_params']
    cwd = os.getcwd()
    for ii in fp_tasks:
        os.chdir(ii)
        sys_data = dpdata.System('POSCAR').data
        ret = make_gaussian_input(sys_data, fp_params)
        with open('input', 'w') as fp:
            fp.write(ret)
        os.chdir(cwd)


def make_fp_vasp(iter_index, jdata):
    # abs path for fp_incar if it exists
    if 'fp_incar' in jdata:
        jdata['fp_incar'] = os.path.abspath(jdata['fp_incar'])
    # get nbands esti if it exists
    if 'fp_nbands_esti_data' in jdata:
        nbe = NBandsEsti(jdata['fp_nbands_esti_data'])
    else:
        nbe = None
    # order is critical!
    # 1, create potcar
    sys_link_fp_vasp_pp(iter_index, jdata)
    # 2, create incar
    make_fp_vasp_incar(iter_index, jdata, nbands_esti = nbe)
    # 3, create kpoints
    make_fp_vasp_kp(iter_index, jdata)
    # 4, copy cvasp
    make_fp_vasp_cp_cvasp(iter_index,jdata)


def make_fp_calculation(iter_index, jdata):
    fp_style = jdata['fp_style']
    if fp_style == 'vasp':
        make_fp_vasp(iter_index, jdata)
    elif fp_style == 'gaussian':
        make_fp_gaussian(iter_index, jdata)
    else :
        raise RuntimeError('unsupported fp_style ' + fp_style)


def make_fp(iter_index, jdata, mdata):
    labeled = jdata.get("labeled", False)
    if labeled:
        make_fp_labeled(iter_index, jdata)
    else:
        make_fp_configs(iter_index, jdata)
        make_fp_calculation(iter_index, jdata)


def run_iter(param_file, machine_file):
    """ init (iter 0): init_pick

    tasks (iter > 0):
    00 make_train (same as generator)
    01 run_train (same as generator)
    02 post_train (same as generator)
    03 make_model_devi
    04 run_model_devi
    05 post_model_devi
    06 make_fp
    07 run_fp (same as generator)
    08 post_fp (same as generator)
    """
    # TODO: function of handling input json should be combined as one function
    try:
        import ruamel
        from monty.serialization import loadfn, dumpfn
        warnings.simplefilter(
            'ignore', ruamel.yaml.error.MantissaNoDotYAML1_1Warning)
        jdata = loadfn(param_file)
        mdata = loadfn(machine_file)
    except Exception:
        with open(param_file, 'r') as fp:
            jdata = json.load(fp)
        with open(machine_file, 'r') as fp:
            mdata = json.load(fp)

    if jdata.get('pretty_print', False):
        fparam = SHORT_CMD+'_' + \
            param_file.split('.')[0]+'.'+jdata.get('pretty_format', 'json')
        dumpfn(jdata, fparam, indent=4)
        fmachine = SHORT_CMD+'_' + \
            machine_file.split('.')[0]+'.'+jdata.get('pretty_format', 'json')
        dumpfn(mdata, fmachine, indent=4)

    if mdata.get('handlers', None):
        if mdata['handlers'].get('smtp', None):
            que = queue.Queue(-1)
            queue_handler = logging.handlers.QueueHandler(que)
            smtp_handler = logging.handlers.SMTPHandler(
                **mdata['handlers']['smtp'])
            listener = logging.handlers.QueueListener(que, smtp_handler)
            dlog.addHandler(queue_handler)
            listener.start()
            
    mdata = convert_mdata(mdata)
    max_tasks = 10000
    numb_task = 9
    record = "record.dpgen"
    iter_rec = [0, -1]
    if os.path.isfile(record):
        with open(record) as frec:
            for line in frec:
                iter_rec = [int(x) for x in line.split()]
        dlog.info("continue from iter %03d task %02d" %
                  (iter_rec[0], iter_rec[1]))

    cont = True
    ii = -1
    while cont:
        ii += 1
        iter_name = make_iter_name(ii)
        sepline(iter_name, '=')
        for jj in range(numb_task):
            if ii * max_tasks + jj <= iter_rec[0] * max_tasks + iter_rec[1]:
                continue
            task_name = "task %02d" % jj
            sepline("{} {}".format(iter_name, task_name), '-')
            jdata['model_devi_jobs'] = [{} for _ in range(ii+1)]
            if ii == 0 and jj < 6:
                if jj == 0:
                    log_iter("init_pick", ii, jj)
                    init_model(ii, jdata, mdata)
                    init_pick(ii, jdata, mdata)
                dlog.info("first iter, skip step 1-5")
            elif jj == 0:
                log_iter("make_train", ii, jj)
                make_train(ii, jdata, mdata)
            elif jj == 1:
                log_iter("run_train", ii, jj)
                #disp = make_dispatcher(mdata['train_machine'])
                run_train(ii, jdata, mdata)
            elif jj == 2:
                log_iter("post_train", ii, jj)
                post_train(ii, jdata, mdata)
            elif jj == 3:
                log_iter("make_model_devi", ii, jj)
                cont = make_model_devi(ii, jdata, mdata)
                if not cont or ii >= jdata.get("stop_iter", ii+1):
                    break
            elif jj == 4:
                log_iter("run_model_devi", ii, jj)
                #disp = make_dispatcher(mdata['model_devi_machine'])
                run_model_devi(ii, jdata, mdata)
            elif jj == 5:
                log_iter("post_model_devi", ii, jj)
                post_model_devi(ii, jdata, mdata)
            elif jj == 6:
                log_iter("make_fp", ii, jj)
                make_fp(ii, jdata, mdata)
            elif jj == 7:
                log_iter("run_fp", ii, jj)
                if jdata.get("labeled", False):
                    dlog.info("already have labeled data, skip run_fp")
                else:
                    #disp = make_dispatcher(mdata['fp_machine'])
                    run_fp(ii, jdata, mdata)
            elif jj == 8:
                log_iter("post_fp", ii, jj)
                if jdata.get("labeled", False):
                    dlog.info("already have labeled data, skip post_fp")
                else:
                    post_fp(ii, jdata)
            else:
                raise RuntimeError("unknown task %d, something wrong" % jj)
            record_iter(record, ii, jj)


def gen_simplify(args):
    if args.PARAM and args.MACHINE:
        if args.debug:
            dlog.setLevel(logging.DEBUG)
        dlog.info("start simplifying")
        run_iter(args.PARAM, args.MACHINE)
        dlog.info("finished")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np                
import os,sys,glob,time           
from deepmd.calculator import DP  
from ase.io import read           

'''
check if structure optimization worked well
if not, this script will generate a fake outcar
'''
                                                                                        
def Get_Element_Num(elements):                        
    '''Using the Atoms.symples to Know Element&Num''' 
    element = []                                      
    ele = {}                                          
    element.append(elements[0])                       
    for x  in elements:                               
        if x not in element :                         
            element.append(x)                         
    for x in element:                                 
        ele[x] = elements.count(x)                    
    return element, ele                               
                                                      
def Write_Contcar(element, ele, lat, pos):                                              
    '''Write CONTCAR'''                                                                 
    f = open('CONTCAR','w')                                                             
    f.write('ASE-DPKit-FAILED-nan\n')                                                
    f.write('1.0\n')                                                                   
    for i in range(3):                                                                  
        f.write('%15.10f %15.10f %15.10f\n' % tuple(lat[i]))                           
    for x in element:                                                                   
        f.write(x + '  ')                                                               
    f.write('\n')                                                                      
    for x in element:                                                                   
        f.write(str(ele[x]) + '  ')                                                     
    f.write('\n')                                                                      
    f.write('Direct\n')                                                                
    na = sum(ele.values())                                                              
    dpos = np.dot(pos,np.linalg.inv(lat))                                               
    for i in range(na):                                                                 
        f.write('%15.10f %15.10f %15.10f\n' % tuple(dpos[i]))                          
                                                                                        
def Write_Outcar(element, ele, volume, lat, pos, ene, force, stress,pstress):           
    '''Write OUTCAR'''                                                                  
    f = open('OUTCAR','w')                                                              
    for x in element:                                                                   
        f.write('VRHFIN =' + str(x) + '\n')                                            
    f.write('ions per type =')                                                          
    for x in element:                                                                   
        f.write('%5d' % ele[x])                                                         
    #f.write('\nvolume of cell :\n')                                                  
    f.write('\nDirection     XX             YY             ZZ             XY             YZ             ZX\n') 
    f.write('in kB')                                                                    
    f.write('%15.6f' % stress[0])                                                       
    f.write('%15.6f' % stress[1])                                                       
    f.write('%15.6f' % stress[2])                                                       
    f.write('%15.6f' % stress[3])                                                       
    f.write('%15.6f' % stress[4])                                                       
    f.write('%15.6f' % stress[5])                                                       
    f.write('\n')                                                                      
    ext_pressure = np.sum(stress[0] + stress[1] + stress[2])/3.0 - pstress              
    f.write('external pressure = %20.6f kB    Pullay stress = %20.6f  kB\n'% (ext_pressure, pstress))
    f.write('volume of cell : %20.6f\n' % volume)                                      
    f.write('direct lattice vectors\n')                                                
    for i in range(3):                                                                  
        f.write('%10.6f %10.6f %10.6f\n' % tuple(lat[i]))                              
    f.write('POSITION                                       TOTAL-FORCE(eV/Angst)\n')  
    f.write('-------------------------------------------------------------------\n')   
    na = sum(ele.values())                                                              
    for i in range(na):                                                                 
        f.write('%15.6f %15.6f %15.6f' % tuple(pos[i]))                                 
        f.write('%15.6f %15.6f %15.6f\n' % tuple(force[i]))                            
    f.write('-------------------------------------------------------------------\n')   
    f.write('energy  without entropy= %20.6f %20.6f\n' % (ene, ene))                
    enthalpy = ene + pstress * volume / 1602.17733                                      
    f.write('enthalpy is  TOTEN    = %20.6f %20.6f\n' % (enthalpy, enthalpy))       
                                                                                                                                
def check():                                                                                                                    
                                                                                                                                
    from deepmd.calculator import DP                                                                                            
    from ase.io import read                                                                                                     
    model_path = sys.argv[1]                                                                                                    
    Model_List = glob.glob('%s/graph*pb'%model_path)                                                                            
    calc = DP(model='%s'%(Model_List[0]))    # init the model before iteration                                                  
                                                                                                                                
    to_be_opti = read('POSCAR')                                                                                                 
    to_be_opti.calc = calc                                                                                                      
    # ---------------------------------                                                                                         
    # for failed outcar                                                                                                         
    atoms_symbols_f = to_be_opti.get_chemical_symbols()                                                                         
    element_f, ele_f = Get_Element_Num(atoms_symbols_f)                                                                         
    atoms_vol_f = to_be_opti.get_volume()                                                                                       
    atoms_stress_f = to_be_opti.get_stress()                                                                                    
    atoms_stress_f = atoms_stress_f/(0.01*0.6242)                                                                               
    atoms_lat_f = to_be_opti.cell                                                                                               
    atoms_pos_f = to_be_opti.positions                                                                                          
    atoms_force_f = to_be_opti.get_forces()                                                                                     
    atoms_ene_f =  610612509                                                                                                    
    # ---------------------------------                                                                                         
    Write_Contcar(element_f, ele_f, atoms_lat_f, atoms_pos_f)                                                                   
    Write_Outcar(element_f, ele_f, atoms_vol_f, atoms_lat_f, atoms_pos_f,atoms_ene_f, atoms_force_f, atoms_stress_f * -10.0, 0) 
 
cwd = os.getcwd()                                   
if not os.path.exists(os.path.join(cwd,'OUTCAR')):  
    check()  



import glob
import json
import os
import re

import numpy as np
from monty.serialization import loadfn, dumpfn

import dpgen.auto_test.lib.vasp as vasp
from dpgen import dlog
from dpgen.auto_test.Property import Property
from dpgen.auto_test.refine import make_refine
from dpgen.auto_test.reproduce import make_repro
from dpgen.auto_test.reproduce import post_repro


class EOS(Property):
    def __init__(self,
                 parameter):
        parameter['reproduce'] = parameter.get('reproduce', False)
        self.reprod = parameter['reproduce']
        if not self.reprod:
            if not ('init_from_suffix' in parameter and 'output_suffix' in parameter):
                self.vol_start = parameter['vol_start']
                self.vol_end = parameter['vol_end']
                self.vol_step = parameter['vol_step']
                parameter['vol_abs'] = parameter.get('vol_abs', False)
                self.vol_abs = parameter['vol_abs']
            parameter['cal_type'] = parameter.get('cal_type', 'relaxation')
            self.cal_type = parameter['cal_type']
            default_cal_setting = {"relax_pos": True,
                                   "relax_shape": True,
                                   "relax_vol": False}
            if 'cal_setting' not in parameter:
                parameter['cal_setting'] = default_cal_setting
            else:
                if "relax_pos" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_pos'] = default_cal_setting['relax_pos']
                if "relax_shape" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_shape'] = default_cal_setting['relax_shape']
                if "relax_vol" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_vol'] = default_cal_setting['relax_vol']
            self.cal_setting = parameter['cal_setting']
        else:
            parameter['cal_type'] = 'static'
            self.cal_type = parameter['cal_type']
            default_cal_setting = {"relax_pos": False,
                                   "relax_shape": False,
                                   "relax_vol": False}
            if 'cal_setting' not in parameter:
                parameter['cal_setting'] = default_cal_setting
            else:
                if "relax_pos" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_pos'] = default_cal_setting['relax_pos']
                if "relax_shape" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_shape'] = default_cal_setting['relax_shape']
                if "relax_vol" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_vol'] = default_cal_setting['relax_vol']
            self.cal_setting = parameter['cal_setting']
            parameter['init_from_suffix'] = parameter.get('init_from_suffix', '00')
            self.init_from_suffix = parameter['init_from_suffix']
        self.parameter = parameter

    def make_confs(self,
                   path_to_work,
                   path_to_equi,
                   refine=False):
        path_to_work = os.path.abspath(path_to_work)
        if os.path.exists(path_to_work):
            dlog.warning('%s already exists' % path_to_work)
        else:
            os.makedirs(path_to_work)
        path_to_equi = os.path.abspath(path_to_equi)

        if 'start_confs_path' in self.parameter and os.path.exists(self.parameter['start_confs_path']):
            init_path_list = glob.glob(os.path.join(self.parameter['start_confs_path'], '*'))
            struct_init_name_list = []
            for ii in init_path_list:
                struct_init_name_list.append(ii.split('/')[-1])
            struct_output_name = path_to_work.split('/')[-2]
            assert struct_output_name in struct_init_name_list
            path_to_equi = os.path.abspath(os.path.join(self.parameter['start_confs_path'],
                                                        struct_output_name, 'relaxation', 'relax_task'))

        cwd = os.getcwd()
        task_list = []
        if self.reprod:
            print('eos reproduce starts')
            if 'init_data_path' not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter['init_data_path'])
            task_list = make_repro(init_data_path, self.init_from_suffix,
                                   path_to_work, self.parameter.get('reprod_last_frame', True))
            os.chdir(cwd)

        else:
            if refine:
                print('eos refine starts')
                task_list = make_refine(self.parameter['init_from_suffix'],
                                        self.parameter['output_suffix'],
                                        path_to_work)
                os.chdir(cwd)

                init_from_path = re.sub(self.parameter['output_suffix'][::-1],
                                        self.parameter['init_from_suffix'][::-1],
                                        path_to_work[::-1], count=1)[::-1]
                task_list_basename = list(map(os.path.basename, task_list))

                for ii in task_list_basename:
                    init_from_task = os.path.join(init_from_path, ii)
                    output_task = os.path.join(path_to_work, ii)
                    os.chdir(output_task)
                    if os.path.isfile('eos.json'):
                        os.remove('eos.json')
                    if os.path.islink('eos.json'):
                        os.remove('eos.json')
                    os.symlink(os.path.relpath(os.path.join(init_from_task, 'eos.json')), 'eos.json')
                os.chdir(cwd)

            else:
                print('gen eos from ' + str(self.vol_start) + ' to ' + str(self.vol_end) + ' by every ' + str(self.vol_step))
                if self.vol_abs : 
                    dlog.info('treat vol_start and vol_end as absolute volume')
                else : 
                    dlog.info('treat vol_start and vol_end as relative volume')
                equi_contcar = os.path.join(path_to_equi, 'CONTCAR')
                if not os.path.exists(equi_contcar):
                    raise RuntimeError("please do relaxation first")
                vol_to_poscar = vasp.poscar_vol(equi_contcar) / vasp.poscar_natoms(equi_contcar)
                self.parameter['scale2equi'] = []

                task_num = 0
                while self.vol_start + self.vol_step * task_num < self.vol_end:
                # for vol in np.arange(int(self.vol_start * 100), int(self.vol_end * 100), int(self.vol_step * 100)):
                    # vol = vol / 100.0
                    vol = self.vol_start + task_num * self.vol_step
                    #task_num = int((vol - self.vol_start) / self.vol_step)
                    output_task = os.path.join(path_to_work, 'task.%06d' % task_num)
                    os.makedirs(output_task, exist_ok=True)
                    os.chdir(output_task)
                    for ii in ['INCAR', 'POTCAR', 'POSCAR.orig', 'POSCAR', 'conf.lmp', 'in.lammps']:
                        if os.path.exists(ii):
                            os.remove(ii)
                    task_list.append(output_task)
                    os.symlink(os.path.relpath(equi_contcar), 'POSCAR.orig')
                    # scale = (vol / vol_to_poscar) ** (1. / 3.)

                    if self.vol_abs :
                        scale = (vol / vol_to_poscar) ** (1. / 3.)
                        eos_params = {'volume': vol, 'scale': scale}
                    else :
                        scale = vol ** (1. / 3.)
                        eos_params = {'volume': vol * vol_to_poscar, 'scale': scale}
                    dumpfn(eos_params, 'eos.json', indent=4)
                    self.parameter['scale2equi'].append(scale)  # 06/22
                    vasp.poscar_scale('POSCAR.orig', 'POSCAR', scale)
                    task_num += 1
                os.chdir(cwd)
        return task_list

    def post_process(self, task_list):
        pass

    def task_type(self):
        return self.parameter['type']

    def task_param(self):
        return self.parameter

    def _compute_lower(self,
                       output_file,
                       all_tasks,
                       all_res):
        output_file = os.path.abspath(output_file)
        res_data = {}
        ptr_data = "conf_dir: " + os.path.dirname(output_file) + "\n"
        if not self.reprod:
            ptr_data += ' VpA(A^3)  EpA(eV)\n'
            for ii in range(len(all_tasks)):
                # vol = self.vol_start + ii * self.vol_step
                vol = loadfn(os.path.join(all_tasks[ii], 'eos.json'))['volume']
                task_result = loadfn(all_res[ii])
                res_data[vol] = task_result['energies'][-1] / sum(task_result['atom_numbs'])
                ptr_data += '%7.3f  %8.4f \n' % (vol, task_result['energies'][-1] / sum(task_result['atom_numbs']))
                # res_data[vol] = all_res[ii]['energy'] / len(all_res[ii]['force'])
                # ptr_data += '%7.3f  %8.4f \n' % (vol, all_res[ii]['energy'] / len(all_res[ii]['force']))

        else:
            if 'init_data_path' not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter['init_data_path'])
            res_data, ptr_data = post_repro(init_data_path, self.parameter['init_from_suffix'],
                                            all_tasks, ptr_data, self.parameter.get('reprod_last_frame', True))

        with open(output_file, 'w') as fp:
            json.dump(res_data, fp, indent=4)

        return res_data, ptr_data



from dargs import Argument, ArgumentEncoder

from dpgen.arginfo import general_mdata_arginfo


def init_bulk_mdata_arginfo() -> Argument:
    """Generate arginfo for dpgen init_bulk mdata.
    
    Returns
    -------
    Argument
        arginfo
    """
    return general_mdata_arginfo("init_bulk_mdata", ("fp",))


def init_surf_mdata_arginfo() -> Argument:
    """Generate arginfo for dpgen init_surf mdata.
    
    Returns
    -------
    Argument
        arginfo
    """
    return general_mdata_arginfo("init_surf_mdata", ("fp",))


def init_reaction_mdata_arginfo() -> Argument:
    """Generate arginfo for dpgen init_reaction mdata.
    
    Returns
    -------
    Argument
        arginfo
    """
    return general_mdata_arginfo("init_reaction_mdata", ("reaxff", "build", "fp"))


def init_reaction_jdata_arginfo() -> Argument:
    """Generate arginfo for dpgen init_reaction jdata.
    
    Returns
    -------
    Argument
        dpgen init_reaction jdata arginfo
    """
    doc_init_reaction = "Generate initial data for reactive systems for small gas-phase molecules, from a ReaxFF NVT MD trajectory."
    doc_type_map = "Type map, which should match types in the initial data. e.g. [\"C\", \"H\", \"O\"]"
    doc_reaxff = "Parameters for ReaxFF NVT MD."
    doc_data = "Path to initial LAMMPS data file. The atom_style should be charge."
    doc_ff = "Path to ReaxFF force field file. Available in the lammps/potentials directory."
    doc_control = "Path to ReaxFF control file."
    doc_temp = "Target Temperature for the NVT MD simulation. Unit: K."
    doc_dt = "Real time for every time step. Unit: fs."
    doc_tau_t = "Time to determine how rapidly the temperature. Unit: fs."
    doc_dump_frep = "Frequency of time steps to collect trajectory."
    doc_nstep = "Total steps to run the ReaxFF MD simulation."
    doc_cutoff = "Cutoff radius to take clusters from the trajectory. Note that only a complete molecule or free radical will be taken."
    doc_dataset_size = "Collected dataset size for each bond type."
    doc_qmkeywords = "Gaussian keywords for first-principle calculations. e.g. force mn15/6-31g** Geom=PrintInputOrient. Note that \"force\" job is necessary to collect data. Geom=PrintInputOrient should be used when there are more than 50 atoms in a cluster."

    return Argument("init_reaction_jdata", dict, [
        Argument("type_map", list, doc=doc_type_map),
        Argument("reaxff", dict, [
            Argument("data", str, doc=doc_data),
            Argument("ff", str, doc=doc_ff),
            Argument("control", str, doc=doc_control),
            Argument("temp", [float, int], doc=doc_temp),
            Argument("dt", [float, int], doc=doc_dt),
            Argument("tau_t", [float, int], doc=doc_tau_t),
            Argument("dump_freq", int, doc=doc_dump_frep),
            Argument("nstep", int, doc=doc_nstep),
        ], doc=doc_reaxff),
        Argument("cutoff", float, doc=doc_cutoff),
        Argument("dataset_size", int, doc=doc_dataset_size),
        Argument("qmkeywords", str, doc=doc_qmkeywords),
    ], doc=doc_init_reaction)



from dpgen.auto_test.VASP import VASP
from dpgen.auto_test.Lammps import Lammps


def make_calculator(inter_parameter,
                    path_to_poscar):
    """
    Make an instance of Task
    """
    inter_type = inter_parameter['type']
    if inter_type == 'vasp':
        return VASP(inter_parameter, path_to_poscar)
    elif inter_type in ['deepmd', 'meam', 'eam_fs', 'eam_alloy']:
        return Lammps(inter_parameter, path_to_poscar)
    #    if inter_type == 'siesta':
    #        return Siesta(inter_parameter, path_to_poscar)
    #        pass
    else:
        raise RuntimeError(f'unsupported interaction {inter_type}')



import os
import sys
import unittest
from pymatgen.core import Structure
from pymatgen.analysis.structure_matcher import StructureMatcher
from monty.serialization import loadfn,dumpfn

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'
from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.mpdb import get_structure
try:
   os.environ['MAPI_KEY']
   exist_key=True
except Exception:
   exist_key=False


def fit(struct0,struct1) :
    m = StructureMatcher() 
    if m.fit(struct0, struct1) :
       return True
    return False

@unittest.skipIf(not exist_key,"skip mpdb")
class TestMpdb(unittest.TestCase):

    def setUp(self):
        if 'MAPI_KEY' in os.environ:
            self.key=os.environ['MAPI_KEY']
        else:
            self.key=None
        self.mpid='mp-141'
        self.st_file=self.mpid+'.vasp'
        self.st0_file=os.path.join('confs/',self.mpid,self.mpid+'.cif')


    def tearDown(self):
        if os.path.exists(self.st_file):
           os.remove(self.st_file)

    def test_get_structure (self):
        st1=get_structure(self.mpid)
        st0=Structure.from_file(self.st0_file)
        self.assertTrue(fit(st0,st1))



#!/usr/bin/env python3

import logging

from monty.serialization import loadfn

from dpgen import dlog
from dpgen.auto_test.common_equi import make_equi, run_equi, post_equi
from dpgen.auto_test.common_prop import make_property, run_property, post_property


#lammps_task_type = ['deepmd', 'meam', 'eam_fs', 'eam_alloy']

def run_task(step, param_file, machine_file=None):
    jdata=loadfn(param_file)
    confs = jdata['structures']
    inter_parameter = jdata['interaction']

    if machine_file:
       mdata=loadfn(machine_file)

    if step == 'make' and 'relaxation' in jdata:
        relax_param = jdata['relaxation']
        make_equi(confs, inter_parameter, relax_param)

    elif step == 'make' and 'properties' in jdata:
        property_list = jdata['properties']
        make_property(confs, inter_parameter, property_list)

    elif step == 'run' and 'relaxation' in jdata:
        if machine_file is None:
           print('Please supply the machine.json, exit now!')  
           return
        run_equi(confs, inter_parameter, mdata)

    elif step == 'run' and 'properties' in jdata:
        if machine_file is None:
           print('Please supply the machine.json, exit now!')  
           return
        property_list = jdata['properties']
        run_property(confs, inter_parameter, property_list, mdata)

    elif step == 'post' and 'relaxation' in jdata:
        post_equi(confs, inter_parameter)

    elif step == 'post' and 'properties' in jdata:
        property_list = jdata['properties']
        post_property(confs, property_list)

    else:
        raise RuntimeError('unknown tasks')

def gen_test(args):
    dlog.info("start auto-testing")
    if args.debug:
        dlog.setLevel(logging.DEBUG)
    run_task(args.TASK, args.PARAM, args.MACHINE)
    dlog.info("finished!")






import os, sys, json, glob, shutil
import dpdata
import numpy as np
import unittest
import dpdata
from monty.serialization import loadfn, dumpfn
from pymatgen.io.vasp import Incar

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'

from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.EOS import EOS


class TestEOS(unittest.TestCase):

    def setUp(self):
        _jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": ".",
                "potcars": {"Li": "vasp_input/POTCAR"}
            },
            "properties": [
                 {
                  "type":         "eos",
                  "skip":  False,
                  "vol_start":    0.8,
                  "vol_end":      1.2,
                  "vol_step":     0.01,
                  "cal_setting": {
                                  "relax_pos": True, 
                                  "relax_shape": True, 
                                  "relax_vol": False, 
                                  "overwrite_interaction":{
                                              "type":    "deepmd", 
                                              "model":   "lammps_input/frozen_model.pb", 
                                              "type_map":{"Al": 0}
                                              }
                                 }
                 }
            ]
        }

        self.equi_path = 'confs/std-fcc/relaxation/relax_task'
        self.source_path = 'equi/vasp'
        self.target_path = 'confs/std-fcc/eos_00'
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata['properties']

        self.eos = EOS(_jdata['properties'][0])

    def tearDown(self):
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual('eos', self.eos.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.eos.task_param())

    def test_make_confs_0(self):

        if not os.path.exists(os.path.join(self.equi_path, 'CONTCAR')):
            with self.assertRaises(RuntimeError):
                self.eos.make_confs(self.target_path, self.equi_path)
        shutil.copy(os.path.join(self.source_path, 'CONTCAR'), os.path.join(self.equi_path, 'CONTCAR'))
        task_list = self.eos.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, 'task.*'))

        incar0 = Incar.from_file(os.path.join('vasp_input', 'INCAR.rlx'))
        incar0['ISIF'] = 4

        for ii in dfm_dirs:
            self.assertTrue(os.path.isfile(os.path.join(ii, 'POSCAR')))
            eos_json_file = os.path.join(ii, 'eos.json')
            self.assertTrue(os.path.isfile(eos_json_file))
            eos_json = loadfn(eos_json_file)
            self.assertEqual(os.path.realpath(os.path.join(ii, 'POSCAR.orig')),
                             os.path.realpath(os.path.join(self.equi_path, 'CONTCAR')))
            sys = dpdata.System(os.path.join(ii, 'POSCAR'))
            natoms = sys.get_natoms()
            self.assertAlmostEqual(eos_json['volume'], np.linalg.det(sys['cells'][0]) / natoms)

    def test_make_confs_1(self):
        self.eos.reprod = True
        shutil.copy(os.path.join(self.source_path, 'CONTCAR'), os.path.join(self.equi_path, 'CONTCAR'))
        with self.assertRaises(RuntimeError):
            self.eos.make_confs(self.target_path, self.equi_path)



import os,sys,json,glob,shutil
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'data'
from .context import coll_abacus_md
from .context import out_dir_name
from .context import abacus_param_file
from .context import setUpModule

class TestCollAbacus(unittest.TestCase):
    def setUp(self):
        with open (abacus_param_file, 'r') as fp :
            jdata = json.load (fp)
        self.odir = out_dir_name(jdata)
        assert os.path.isdir('out_data_02_md_abacus'), 'out data for post fp vasp should exist'
        if os.path.isdir(self.odir) :
            shutil.rmtree(self.odir)
        shutil.copytree('out_data_02_md_abacus', self.odir)
        self.ref_coord = np.reshape(np.genfromtxt("abacus.out/coord.raw"), [8, 5, 3])
        self.ref_cell = np.reshape(np.genfromtxt("abacus.out/box.raw"), [8, 3, 3])
        self.ref_e = np.reshape(np.genfromtxt("abacus.out/energy.raw"), [8, ])
        self.ref_f = np.reshape(np.genfromtxt("abacus.out/force.raw"), [8, 5, 3])
        self.ref_v = np.reshape(np.genfromtxt("abacus.out/virial.raw"), [8, 3, 3])
    def tearDown(self):
        #print("escape.")
        shutil.rmtree(self.odir)

    def test_coll(self):

        with open (abacus_param_file, 'r') as fp :
            jdata = json.load (fp)
        jdata['out_dir'] = self.odir
        print(os.getcwd())
        coll_abacus_md(jdata)
        
        sys = dpdata.LabeledSystem(self.odir + '/02.md/sys-0004-0001/deepmd//', fmt = 'deepmd/raw')
        self.assertEqual(sys.get_nframes(), 8)
            
        for ff in range(8) :
            self.assertAlmostEqual(self.ref_e[ff], sys.data['energies'][ff])
        for ff in range(8) :
            for ii in range(5) :
                for dd in range(3) :
                    self.assertAlmostEqual(self.ref_coord[ff][ii][dd], 
                                           sys.data['coords'][ff][ii][dd])
                    self.assertAlmostEqual(self.ref_f[ff][ii][dd], 
                                           sys.data['forces'][ff][ii][dd])
        for ff in range(8):
            for ii in range(3) :
                for jj in range(3) :
                    self.assertAlmostEqual(self.ref_v[ff][ii][jj], 
                                           sys.data['virials'][ff][ii][jj], places = 5)
                    self.assertAlmostEqual(self.ref_cell[ff][ii][jj], 
                                           sys.data['cells'][ff][ii][jj])
                    
if __name__ == '__main__':
    unittest.main()
        

    



from abc import ABC, abstractmethod


class Task(ABC):
    @abstractmethod
    def __init__(self,
                 inter_parameter,
                 path_to_poscar):
        """
        Constructor

        Parameters
        ----------
        inter_parameter : dict
                A dict that specifies the interaction.
        path_to_poscar : str
                The path to POSCAR. Indicating in which system the task will be initialized.
        """
        pass

    @abstractmethod
    def make_potential_files(self,
                             output_dir):
        """
        Prepare potential files for a computational task.
        For example, the VASP prepares POTCAR. 
        DeePMD prepares frozen model(s).
        IMPORTANT: Interaction should be stored in output_dir/inter.json

        Parameters
        ----------
        output_dir : str
                The directory storing the potential files.
        Outputs
        -------
        inter.json: output file
                The task information is stored in `output_dir/inter.json`
        """
        pass

    @abstractmethod
    def make_input_file(self,
                        output_dir,
                        task_type,
                        task_param):
        """
        Prepare input files for a computational task
        For example, the VASP prepares INCAR.
        LAMMPS (including DeePMD, MEAM...) prepares in.lammps.

        Parameters
        ----------
        output_dir : str
                The directory storing the input files.
        task_type : str
                Can be
                - "relaxation:": structure relaxation
                - "static": static computation calculates the energy, force... of a strcture
        task_parame: dict
                The parameters of the task.
                For example the VASP interaction can be provided with
                { "ediff": 1e-6, "ediffg": 1e-5 }
        """
        pass

    @abstractmethod
    def compute(self,
                output_dir):
        """
        Compute output of the task. 
        IMPORTANT: The output configuration should be converted and stored in a CONTCAR file.

        Parameters
        ----------
        output_dir : str
                The directory storing the input and output files.

        Returns
        -------
        result_dict: dict
                A dict that storing the result. For example:
                { "energy": xxx, "force": [xxx] }

        Outputs
        -------
        CONTCAR: output file
                The output configuration is converted to CONTCAR and stored in the `output_dir`
        """
        pass

    @property
    @staticmethod
    @abstractmethod
    def forward_files(self):
        """
        Return forward files.
        """
        pass

    @property
    @staticmethod
    @abstractmethod
    def forward_common_files(self):
        """
        Return forward common files.
        """
        pass

    @property
    @staticmethod
    @abstractmethod
    def backward_files(self):
        """
        Return backward files.
        """
        pass



import os
from dpgen import dlog
from dpgen.util import sepline
import dpgen.auto_test.lib.vasp as vasp
from dpgen.auto_test.Task import Task
from dpgen.generator.lib.vasp import incar_upper
from dpdata import LabeledSystem
from monty.serialization import dumpfn
from pymatgen.io.vasp import Incar, Kpoints
from pymatgen.core.structure import Structure


class VASP(Task):
    def __init__(self,
                 inter_parameter,
                 path_to_poscar):
        self.inter = inter_parameter
        self.inter_type = inter_parameter['type']
        self.incar = inter_parameter['incar']
        self.potcar_prefix = inter_parameter.get('potcar_prefix', '')
        self.potcars = inter_parameter['potcars']
        self.path_to_poscar = path_to_poscar

    def make_potential_files(self,
                             output_dir):
        potcar_not_link_list = ['vacancy', 'interstitial']
        task_type = output_dir.split('/')[-2].split('_')[0]

        ele_pot_list = [key for key in self.potcars.keys()]
        poscar = os.path.abspath(os.path.join(output_dir, 'POSCAR'))
        pos_str = Structure.from_file(poscar)
        ele_pos_list_tmp = list(ii.as_dict()['element'] for ii in pos_str.species)

        ele_pos_list = [ele_pos_list_tmp[0]]
        for ii in range(1, len(ele_pos_list_tmp)):
            if not ele_pos_list_tmp[ii] == ele_pos_list_tmp[ii - 1]:
                ele_pos_list.append(ele_pos_list_tmp[ii])

        if task_type in potcar_not_link_list:
            with open(os.path.join(output_dir, 'POTCAR'), 'w') as fp:
                for ii in ele_pos_list:
                    for jj in ele_pot_list:
                        if ii == jj:
                            with open(os.path.join(self.potcar_prefix, self.potcars[jj]), 'r') as fin:
                                for line in fin:
                                    print(line.strip('\n'), file=fp)

        else:
            if not os.path.isfile(os.path.join(output_dir, '../POTCAR')):
                with open(os.path.join(output_dir, '../POTCAR'), 'w') as fp:
                    for ii in ele_pos_list:
                        for jj in ele_pot_list:
                            if ii == jj:
                                with open(os.path.join(self.potcar_prefix, self.potcars[jj]), 'r') as fin:
                                    for line in fin:
                                        print(line.strip('\n'), file=fp)
            cwd = os.getcwd()
            os.chdir(output_dir)
            if not os.path.islink('POTCAR'):
                os.symlink('../POTCAR', 'POTCAR')
            elif not '../POTCAR' == os.readlink('POTCAR'):
                os.remove('POTCAR')
                os.symlink('../POTCAR', 'POTCAR')
            os.chdir(cwd)

        dumpfn(self.inter, os.path.join(output_dir, 'inter.json'), indent=4)

    def make_input_file(self,
                        output_dir,
                        task_type,
                        task_param):
        sepline(ch=output_dir)
        dumpfn(task_param, os.path.join(output_dir, 'task.json'), indent=4)

        assert (os.path.exists(self.incar)), 'no INCAR file for relaxation'
        relax_incar_path = os.path.abspath(self.incar)
        incar_relax = incar_upper(Incar.from_file(relax_incar_path))

        # deal with relaxation

        cal_type = task_param['cal_type']
        cal_setting = task_param['cal_setting']

        # user input INCAR for property calculation
        if 'input_prop' in cal_setting and os.path.isfile(cal_setting['input_prop']):
            incar_prop = os.path.abspath(cal_setting['input_prop'])
            incar = incar_upper(Incar.from_file(incar_prop))

        # revise INCAR based on the INCAR provided in the "interaction"
        else:
            incar = incar_relax
            if cal_type == 'relaxation':
                relax_pos = cal_setting['relax_pos']
                relax_shape = cal_setting['relax_shape']
                relax_vol = cal_setting['relax_vol']
                if [relax_pos, relax_shape, relax_vol] == [True, False, False]:
                    isif = 2
                elif [relax_pos, relax_shape, relax_vol] == [True, True, True]:
                    isif = 3
                elif [relax_pos, relax_shape, relax_vol] == [True, True, False]:
                    isif = 4
                elif [relax_pos, relax_shape, relax_vol] == [False, True, False]:
                    isif = 5
                elif [relax_pos, relax_shape, relax_vol] == [False, True, True]:
                    isif = 6
                elif [relax_pos, relax_shape, relax_vol] == [False, False, True]:
                    isif = 7
                elif [relax_pos, relax_shape, relax_vol] == [False, False, False]:
                    nsw = 0
                    isif = 2
                    if not ('NSW' in incar and incar.get('NSW') == nsw):
                        dlog.info("%s setting NSW to %d" % (self.make_input_file.__name__, nsw))
                        incar['NSW'] = nsw
                else:
                    raise RuntimeError("not supported calculation setting for VASP")

                if not ('ISIF' in incar and incar.get('ISIF') == isif):
                    dlog.info("%s setting ISIF to %d" % (self.make_input_file.__name__, isif))
                    incar['ISIF'] = isif

            elif cal_type == 'static':
                nsw = 0
                if not ('NSW' in incar and incar.get('NSW') == nsw):
                    dlog.info("%s setting NSW to %d" % (self.make_input_file.__name__, nsw))
                    incar['NSW'] = nsw

            else:
                raise RuntimeError("not supported calculation type for VASP")

            if 'ediff' in cal_setting:
                dlog.info("%s setting EDIFF to %s" % (self.make_input_file.__name__, cal_setting['ediff']))
                incar['EDIFF'] = cal_setting['ediff']

            if 'ediffg' in cal_setting:
                dlog.info("%s setting EDIFFG to %s" % (self.make_input_file.__name__, cal_setting['ediffg']))
                incar['EDIFFG'] = cal_setting['ediffg']

            if 'encut' in cal_setting:
                dlog.info("%s setting ENCUT to %s" % (self.make_input_file.__name__, cal_setting['encut']))
                incar['ENCUT'] = cal_setting['encut']

            if 'kspacing' in cal_setting:
                dlog.info("%s setting KSPACING to %s" % (self.make_input_file.__name__, cal_setting['kspacing']))
                incar['KSPACING'] = cal_setting['kspacing']

            if 'kgamma' in cal_setting:
                dlog.info("%s setting KGAMMA to %s" % (self.make_input_file.__name__, cal_setting['kgamma']))
                incar['KGAMMA'] = cal_setting['kgamma']

        try:
            kspacing = incar.get('KSPACING')
        except KeyError:
            raise RuntimeError("KSPACING must be given in INCAR")

        if 'KGAMMA' in incar:
            kgamma = incar.get('KGAMMA')
        else:
            kgamma = False

        incar.write_file(os.path.join(output_dir, '../INCAR'))
        cwd = os.getcwd()
        os.chdir(output_dir)
        if not os.path.islink('INCAR'):
            os.symlink('../INCAR', 'INCAR')
        elif not '../INCAR' == os.readlink('INCAR'):
            os.remove('INCAR')
            os.symlink('../INCAR', 'INCAR')
        os.chdir(cwd)
        ret = vasp.make_kspacing_kpoints(self.path_to_poscar, kspacing, kgamma)
        kp = Kpoints.from_string(ret)
        kp.write_file(os.path.join(output_dir, "KPOINTS"))

    def compute(self,
                output_dir):
        outcar = os.path.join(output_dir, 'OUTCAR')
        if not os.path.isfile(outcar):
            dlog.warning("cannot find OUTCAR in " + output_dir + " skip")
            return None
        else:
            ls = LabeledSystem(outcar)
            stress = []
            with open(outcar, 'r') as fin:
                lines = fin.read().split('\n')
            for line in lines:
                if 'in kB' in line:
                    stress_xx = float(line.split()[2])
                    stress_yy = float(line.split()[3])
                    stress_zz = float(line.split()[4])
                    stress_xy = float(line.split()[5])
                    stress_yz = float(line.split()[6])
                    stress_zx = float(line.split()[7])
                    stress.append([])
                    stress[-1].append([stress_xx, stress_xy, stress_zx])
                    stress[-1].append([stress_xy, stress_yy, stress_yz])
                    stress[-1].append([stress_zx, stress_yz, stress_zz])

            outcar_dict = ls.as_dict()
            outcar_dict['data']['stress'] = {"@module": "numpy", "@class": "array", "dtype": "float64", "data": stress}

            return outcar_dict

    def forward_files(self, property_type='relaxation'):
        return ['INCAR', 'POSCAR', 'KPOINTS', 'POTCAR']

    def forward_common_files(self, property_type='relaxation'):
        potcar_not_link_list = ['vacancy', 'interstitial']
        if property_type == 'elastic':
            return ['INCAR', 'KPOINTS', 'POTCAR']
        elif property_type in potcar_not_link_list:
            return ['INCAR']
        else:
            return ['INCAR', 'POTCAR']

    def backward_files(self, property_type='relaxation'):
        return ['OUTCAR', 'outlog', 'CONTCAR', 'OSZICAR', 'XDATCAR']



import glob
import os
import re


def make_refine(init_from_suffix, output_suffix, path_to_work):
    cwd = os.getcwd()
    init_from = re.sub(output_suffix[::-1], init_from_suffix[::-1], path_to_work[::-1], count=1)[::-1]
    if not os.path.exists(init_from):
        raise FileNotFoundError("the initial directory does not exist for refine")

    output = path_to_work
    init_from_task_tot = glob.glob(os.path.join(init_from, 'task.[0-9]*[0-9]'))

    task_num = len(init_from_task_tot)

    task_list = []
    for ii in range(task_num):
        output_task = os.path.join(output, 'task.%06d' % ii)
        os.makedirs(output_task, exist_ok=True)
        os.chdir(output_task)
        for jj in ['INCAR', 'POTCAR', 'POSCAR.orig', 'POSCAR', 'conf.lmp', 'in.lammps']:
            if os.path.exists(jj):
                os.remove(jj)
        task_list.append(output_task)
        init_from_task = os.path.join(init_from, 'task.%06d' % ii)
        if not os.path.exists(init_from_task):
            raise FileNotFoundError("the initial task directory does not exist for refine")
        contcar = os.path.join(init_from_task, 'CONTCAR')
        init_poscar = os.path.join(init_from_task, 'POSCAR')
        if os.path.exists(contcar):
            os.symlink(os.path.relpath(contcar), 'POSCAR')
        elif os.path.exists(init_poscar):
            os.symlink(os.path.relpath(init_poscar), 'POSCAR')
        else:
            raise FileNotFoundError("no CONTCAR or POSCAR in the init_from directory")
    os.chdir(cwd)

    return task_list



#!/usr/bin/env python
# coding: utf-8
from typing import Union, List
from pathlib import Path

from dargs import Argument

from dpgen import dlog

"""
some common utilities for generator, auto_test and data
"""

# constants define
MaxLength=70

def sepline(ch='-',sp='-',screen=False):
    r'''
    seperate the output by '-'
    '''
    if screen:
       print(ch.center(MaxLength,sp))
    else:
       dlog.info(ch.center(MaxLength,sp))

def box_center(ch='',fill=' ',sp="|"):
    r'''
    put the string at the center of |  |
    '''
    strs=ch.center(Len,fill)
    dlog.info(sp+strs[1:len(strs)-1:]+sp)


def expand_sys_str(root_dir: Union[str, Path]) -> List[str]:
    """Recursively iterate over directories taking those that contain `type.raw` file.

    Parameters
    ----------
    root_dir : Union[str, Path]
        starting directory

    Returns
    -------
    List[str]
        list of string pointing to system directories
    """
    root_dir = Path(root_dir)
    matches = [str(d) for d in root_dir.rglob("*") if (d / "type.raw").is_file()]
    if (root_dir / "type.raw").is_file():
        matches.append(str(root_dir))
    return matches

def normalize(arginfo: Argument, data: dict, strict_check: bool = True) -> dict:
    """Normalize and check input data.

    Parameters
    ----------
    arginfo : dargs.Argument
        argument information
    data : dict
        input data
    strict_check : bool, default=True
        strict check data or not

    Returns
    -------
    dict
        normalized data
    """
    data = arginfo.normalize_value(data, trim_pattern="_*")
    arginfo.check_value(data, strict=strict_check)
    return data



from typing import List
from dargs import Argument, Variant

from dpgen.arginfo import general_mdata_arginfo
from dpgen.generator.arginfo import (
    basic_args,
    data_args,
    training_args,
    fp_style_vasp_args,
    fp_style_gaussian_args,
)


def general_simplify_arginfo() -> Argument:
    """General simplify arginfo.

    Returns
    -------
    Argument
        arginfo
    """
    doc_labeled = "If true, the initial data is labeled."
    doc_pick_data = "Path to the directory with the pick data with the deepmd/npy format. Systems are detected recursively."
    doc_init_pick_number = "The number of initial pick data."
    doc_iter_pick_number = "The number of pick data in each iteration."
    doc_model_devi_f_trust_lo = "The lower bound of forces for the selection for the model deviation."
    doc_model_devi_f_trust_hi = "The higher bound of forces for the selection for the model deviation."

    return [
        Argument("labeled", bool, optional=True, default=False, doc=doc_labeled),
        Argument("pick_data", str, doc=doc_pick_data),
        Argument("init_pick_number", int, doc=doc_init_pick_number),
        Argument("iter_pick_number", int, doc=doc_iter_pick_number),
        Argument("model_devi_f_trust_lo", float, optional=False, doc=doc_model_devi_f_trust_lo),
        Argument("model_devi_f_trust_hi", float, optional=False, doc=doc_model_devi_f_trust_hi),
    ]


def fp_style_variant_type_args() -> Variant:
    """Generate variant for fp style variant type.
    
    Returns
    -------
    Variant
        variant for fp style
    """
    doc_fp_style = 'Software for First Principles, if `labeled` is false. Options include “vasp”, “gaussian” up to now.'
    doc_fp_style_none = 'No fp.'
    doc_fp_style_vasp = 'VASP.'
    doc_fp_style_gaussian = 'Gaussian. The command should be set as `g16 < input`.'

    return Variant("fp_style", [
        Argument("none", dict, doc=doc_fp_style_none),
        # simplify use the same fp method as run
        Argument("vasp", dict, fp_style_vasp_args(), doc=doc_fp_style_vasp),
        Argument("gaussian", dict, fp_style_gaussian_args(),
                 doc=doc_fp_style_gaussian),
    ],
        optional=True,
        default_tag="none",
        doc=doc_fp_style)


def fp_args() -> List[Argument]:
    """Generate arginfo for fp.

    Returns
    -------
    List[Argument]
        arginfo
    """
    doc_fp_task_max = 'Maximum of structures to be calculated in 02.fp of each iteration.'
    doc_fp_task_min = 'Minimum of structures to be calculated in 02.fp of each iteration.'

    return [
        Argument("fp_task_max", int, optional=True, doc=doc_fp_task_max),
        Argument("fp_task_min", int, optional=True, doc=doc_fp_task_min),
    ]


def simplify_jdata_arginfo() -> Argument:
    """Generate arginfo for dpgen simplify jdata.

    Returns
    -------
    Argument
        arginfo
    """
    doc_run_jdata = "Parameters for simplify.json, the first argument of `dpgen simplify`."
    return Argument("simplify_jdata",
                    dict,
                    sub_fields=[
                        *basic_args(),
                        # TODO: we may remove sys_configs; it is required in train method
                        *data_args(),
                        *general_simplify_arginfo(),
                        # simplify use the same training method as run
                        *training_args(),
                        *fp_args(),
                    ],
                    sub_variants=[
                        fp_style_variant_type_args(),
                    ],
                    doc=doc_run_jdata,
                    )


def simplify_mdata_arginfo() -> Argument:
    """Generate arginfo for dpgen simplify mdata.

    Returns
    -------
    Argument
        arginfo
    """
    return general_mdata_arginfo("simplify_mdata", ("train", "model_devi", "fp"))



import os
from dpgen import dlog
from pymatgen.ext.matproj import MPRester, MPRestError
from pymatgen.core import Structure

web="materials.org"

def check_apikey():
    try:
      apikey=os.environ['MAPI_KEY']
    except KeyError:
       print("You have to get a MAPI_KEY from "+web)
       print("and execute following command:")
       print('echo "export MAPI_KEY=yourkey">> ~/.bashrc')
       print("source ~/.bashrc")
       os._exit(0)
    try:
      return MPRester(apikey)
    except MPRestError:
       dlog.info("MPRester Error, you need to prepare POSCAR manually")
       os._exit(0)

def get_structure(mp_id):
    mpr=check_apikey()
    return mpr.get_structure_by_material_id(mp_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os,sys,glob,time                                                                 
import numpy as np                                                                      
from ase.io import read                                                                 
from ase.optimize import BFGS,QuasiNewton,LBFGS                                         
from ase.constraints import UnitCellFilter, ExpCellFilter
from deepmd.calculator import DP                                                        
'''
structure optimization with DP model and ASE
PSTRESS and fmax should exist in input.dat
'''
                                                                                        
def Get_Element_Num(elements):                                                          
    '''Using the Atoms.symples to Know Element&Num'''                                   
    element = []                                                                        
    ele = {}                                                                            
    element.append(elements[0])                                                         
    for x  in elements:                                                                 
        if x not in element :                                                           
            element.append(x)                                                           
    for x in element:                                                                   
        ele[x] = elements.count(x)                                                      
    return element, ele                                                                 
                                                                                        
def Write_Contcar(element, ele, lat, pos):                                              
    '''Write CONTCAR'''                                                                 
    f = open('CONTCAR','w')                                                             
    f.write('ASE-DPKit-Optimization\n')                                                
    f.write('1.0\n')                                                                   
    for i in range(3):                                                                  
        f.write('%15.10f %15.10f %15.10f\n' % tuple(lat[i]))                           
    for x in element:                                                                   
        f.write(x + '  ')                                                               
    f.write('\n')                                                                      
    for x in element:                                                                   
        f.write(str(ele[x]) + '  ')                                                     
    f.write('\n')                                                                      
    f.write('Direct\n')                                                                
    na = sum(ele.values())                                                              
    dpos = np.dot(pos,np.linalg.inv(lat))                                               
    for i in range(na):                                                                 
        f.write('%15.10f %15.10f %15.10f\n' % tuple(dpos[i]))                          
                                                                                        
def Write_Outcar(element, ele, volume, lat, pos, ene, force, stress,pstress):           
    '''Write OUTCAR'''                                                                  
    f = open('OUTCAR','w')                                                              
    for x in element:                                                                   
        f.write('VRHFIN =' + str(x) + '\n')                                            
    f.write('ions per type =')                                                          
    for x in element:                                                                   
        f.write('%5d' % ele[x])                                                         
    #f.write('\nvolume of cell :\n')                                                  
    f.write('\nDirection     XX             YY             ZZ             XY             YZ             ZX\n') 
    f.write('in kB')                                                                    
    f.write('%15.6f' % stress[0])                                                       
    f.write('%15.6f' % stress[1])                                                       
    f.write('%15.6f' % stress[2])                                                       
    f.write('%15.6f' % stress[3])                                                       
    f.write('%15.6f' % stress[4])                                                       
    f.write('%15.6f' % stress[5])                                                       
    f.write('\n')                                                                      
    ext_pressure = np.sum(stress[0] + stress[1] + stress[2])/3.0 - pstress              
    f.write('external pressure = %20.6f kB    Pullay stress = %20.6f  kB\n'% (ext_pressure, pstress))
    f.write('volume of cell : %20.6f\n' % volume)                                      
    f.write('direct lattice vectors\n')                                                
    for i in range(3):                                                                  
        f.write('%10.6f %10.6f %10.6f\n' % tuple(lat[i]))                              
    f.write('POSITION                                       TOTAL-FORCE(eV/Angst)\n')  
    f.write('-------------------------------------------------------------------\n')   
    na = sum(ele.values())                                                              
    for i in range(na):                                                                 
        f.write('%15.6f %15.6f %15.6f' % tuple(pos[i]))                                 
        f.write('%15.6f %15.6f %15.6f\n' % tuple(force[i]))                            
    f.write('-------------------------------------------------------------------\n')   
    f.write('energy  without entropy= %20.6f %20.6f\n' % (ene, ene/na))                
    enthalpy = ene + pstress * volume / 1602.17733                                      
    f.write('enthalpy is  TOTEN    = %20.6f %20.6f\n' % (enthalpy, enthalpy/na))       
                                                                                        
def Write_calylog(templog):                                                             
    '''For Write Evolve Structures Log into caly.log'''                                 
    f = open('opt.log','a+')                                                           
    f.write(templog+'\n')                                                              
    f.close()                                                                           
def read_stress_fmax():
    pstress = 0
    fmax = 0.01
    #assert os.path.exists('./input.dat'), 'input.dat does not exist!'
    try:
        f = open('input.dat','r')
    except:
        assert os.path.exists('../input.dat'),' now we are in %s, do not find ../input.dat'%(os.getcwd())
        f = open('../input.dat','r')
    lines = f.readlines()
    f.close()
    for line in lines:
        if line[0] == '#':
            continue
        if 'PSTRESS' in line or 'pstress' in line:
            pstress = float(line.split('=')[1])
        if 'fmax' in line:
            fmax = float(line.split('=')[1])
    return fmax,pstress
                                                                                        
                                                                                        
def run_opt(fmax,stress):                                                                          
    '''Using the ASE&DP to Optimize Configures'''                                       
    ''' > 600 Steps Called Failure Optimization'''                                      
    
    os.system('mv OUTCAR OUTCAR-last')
    is_dpgen = True
    nn = 1
    try:
        model_path = sys.argv[1]                                                            
        Model_List = glob.glob('%s/graph*pb'%model_path)                                    
        calc = DP(model=Model_List[0])    # init the model before iteration          
    except:
        assert os.path.exists('graph.pb'), 'did not found graph.pb in this directory %s, or did you forget to add args?'%(os.getcwd())
        calc = DP(model='graph.pb')    # init the model before iteration          
        is_dpgen = False
        nn = 3
                                                                                        
    print('Start to Optimize Structures by DP----------')                               
                                                                                        
    Opt_Step = 600                                                                      
    start = time.time()                                                                 
    # pstress kbar
    pstress = stress
    # kBar to eV/A^3
    # 1 eV/A^3 = 160.21766028 GPa
    # 1 / 160.21766028 ~ 0.006242
    aim_stress = 1.0 * pstress* 0.01 * 0.6242  / 10.0 
    to_be_opti = read('POSCAR')                                                         
    to_be_opti.calc = calc                                                              
    ucf = UnitCellFilter(to_be_opti, scalar_pressure=aim_stress)
    atoms_vol_2 = to_be_opti.get_volume()                                               
    for i in range(nn):                                                                  
        if is_dpgen:
            opt = LBFGS(ucf,trajectory='traj.traj')                                                         
        else:
            opt = LBFGS(ucf)                                                         
        #opt = QuasiNewton(to_be_opti)                                                  
        #opt = BFGS(to_be_opti)                                                         
        #opt = BFGS(to_be_opti,trajectory='traj.traj',logfile='opt.log')                
        opt.run(fmax=fmax,steps=Opt_Step)                       
                                                                                        
        atoms_lat = to_be_opti.cell                                                     
        atoms_pos = to_be_opti.positions                                                
        atoms_force = to_be_opti.get_forces()                                           
        atoms_stress = to_be_opti.get_stress()                                          
        # eV/A^3 to GPa
        atoms_stress = atoms_stress/(0.01*0.6242)
        #atoms_num = to_be_opti.get_atomic_numbers()                                    
        atoms_symbols = to_be_opti.get_chemical_symbols()                               
        #atoms_formula = to_be_opti.get_chemical_formula()                              
        atoms_ene = to_be_opti.get_potential_energy()                                   
        atoms_vol = to_be_opti.get_volume()                                             
                                                                                        
        element, ele = Get_Element_Num(atoms_symbols)                                   
                                                                                        
        Write_Contcar(element, ele, atoms_lat, atoms_pos)                               
        Write_Outcar(element, ele, atoms_vol, atoms_lat, atoms_pos,atoms_ene, atoms_force, atoms_stress * -10.0, pstress)                   
                                                                                        
                                                                                        
    stop = time.time()                                                                  
    _cwd = os.getcwd()                                                                  
    _cwd = os.path.basename(_cwd)                                                       
    print('%s is done, time: %s' % (_cwd,stop-start))                                   

def run():
    fmax, stress = read_stress_fmax()
    run_opt(fmax, stress)                                                                               

if __name__=='__main__':
   run()



import glob
import json
import os
import re

import numpy as np
from monty.serialization import loadfn, dumpfn
from pymatgen.core.structure import Structure
from pymatgen.core.surface import generate_all_slabs

import dpgen.auto_test.lib.vasp as vasp
from dpgen import dlog
from dpgen.auto_test.Property import Property
from dpgen.auto_test.refine import make_refine
from dpgen.auto_test.reproduce import make_repro
from dpgen.auto_test.reproduce import post_repro


class Surface(Property):
    def __init__(self,
                 parameter):
        parameter['reproduce'] = parameter.get('reproduce', False)
        self.reprod = parameter['reproduce']
        if not self.reprod:
            if not ('init_from_suffix' in parameter and 'output_suffix' in parameter):
                self.min_slab_size = parameter['min_slab_size']
                self.min_vacuum_size = parameter['min_vacuum_size']
                parameter['pert_xz'] = parameter.get('pert_xz', 0.01)
                self.pert_xz = parameter['pert_xz']
                default_max_miller = 2
                parameter['max_miller'] = parameter.get('max_miller', default_max_miller)
                self.miller = parameter['max_miller']
            parameter['cal_type'] = parameter.get('cal_type', 'relaxation')
            self.cal_type = parameter['cal_type']
            default_cal_setting = {"relax_pos": True,
                                   "relax_shape": True,
                                   "relax_vol": False}
            if 'cal_setting' not in parameter:
                parameter['cal_setting'] = default_cal_setting
            else:
                if "relax_pos" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_pos'] = default_cal_setting['relax_pos']
                if "relax_shape" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_shape'] = default_cal_setting['relax_shape']
                if "relax_vol" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_vol'] = default_cal_setting['relax_vol']
            self.cal_setting = parameter['cal_setting']
        else:
            parameter['cal_type'] = 'static'
            self.cal_type = parameter['cal_type']
            default_cal_setting = {"relax_pos": False,
                                   "relax_shape": False,
                                   "relax_vol": False}
            if 'cal_setting' not in parameter:
                parameter['cal_setting'] = default_cal_setting
            else:
                if "relax_pos" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_pos'] = default_cal_setting['relax_pos']
                if "relax_shape" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_shape'] = default_cal_setting['relax_shape']
                if "relax_vol" not in parameter['cal_setting']:
                    parameter['cal_setting']['relax_vol'] = default_cal_setting['relax_vol']
            self.cal_setting = parameter['cal_setting']
            parameter['init_from_suffix'] = parameter.get('init_from_suffix', '00')
            self.init_from_suffix = parameter['init_from_suffix']
        self.parameter = parameter

    def make_confs(self,
                   path_to_work,
                   path_to_equi,
                   refine=False):
        path_to_work = os.path.abspath(path_to_work)
        if os.path.exists(path_to_work):
            dlog.warning('%s already exists' % path_to_work)
        else:
            os.makedirs(path_to_work)
        path_to_equi = os.path.abspath(path_to_equi)

        if 'start_confs_path' in self.parameter and os.path.exists(self.parameter['start_confs_path']):
            init_path_list = glob.glob(os.path.join(self.parameter['start_confs_path'], '*'))
            struct_init_name_list = []
            for ii in init_path_list:
                struct_init_name_list.append(ii.split('/')[-1])
            struct_output_name = path_to_work.split('/')[-2]
            assert struct_output_name in struct_init_name_list
            path_to_equi = os.path.abspath(os.path.join(self.parameter['start_confs_path'],
                                                        struct_output_name, 'relaxation', 'relax_task'))

        task_list = []
        cwd = os.getcwd()

        if self.reprod:
            print('surface reproduce starts')
            if 'init_data_path' not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter['init_data_path'])
            task_list = make_repro(init_data_path, self.init_from_suffix,
                                   path_to_work, self.parameter.get('reprod_last_frame', True))
            os.chdir(cwd)

        else:
            if refine:
                print('surface refine starts')
                task_list = make_refine(self.parameter['init_from_suffix'],
                                        self.parameter['output_suffix'],
                                        path_to_work)
                os.chdir(cwd)
                # record miller
                init_from_path = re.sub(self.parameter['output_suffix'][::-1],
                                        self.parameter['init_from_suffix'][::-1],
                                        path_to_work[::-1], count=1)[::-1]
                task_list_basename = list(map(os.path.basename, task_list))

                for ii in task_list_basename:
                    init_from_task = os.path.join(init_from_path, ii)
                    output_task = os.path.join(path_to_work, ii)
                    os.chdir(output_task)
                    if os.path.isfile('miller.json'):
                        os.remove('miller.json')
                    if os.path.islink('miller.json'):
                        os.remove('miller.json')
                    os.symlink(os.path.relpath(os.path.join(init_from_task, 'miller.json')), 'miller.json')
                os.chdir(cwd)

            else:
                equi_contcar = os.path.join(path_to_equi, 'CONTCAR')
                if not os.path.exists(equi_contcar):
                    raise RuntimeError("please do relaxation first")
                ptypes = vasp.get_poscar_types(equi_contcar)
                # gen structure
                ss = Structure.from_file(equi_contcar)
                # gen slabs
                all_slabs = generate_all_slabs(ss, self.miller, self.min_slab_size, self.min_vacuum_size)

                os.chdir(path_to_work)
                if os.path.isfile('POSCAR'):
                    os.remove('POSCAR')
                if os.path.islink('POSCAR'):
                    os.remove('POSCAR')
                os.symlink(os.path.relpath(equi_contcar), 'POSCAR')
                #           task_poscar = os.path.join(output, 'POSCAR')
                for ii in range(len(all_slabs)):
                    output_task = os.path.join(path_to_work, 'task.%06d' % ii)
                    os.makedirs(output_task, exist_ok=True)
                    os.chdir(output_task)
                    for jj in ['INCAR', 'POTCAR', 'POSCAR', 'conf.lmp', 'in.lammps']:
                        if os.path.exists(jj):
                            os.remove(jj)
                    task_list.append(output_task)
                    print("# %03d generate " % ii, output_task, " \t %d atoms" % len(all_slabs[ii].sites))
                    # make confs
                    all_slabs[ii].to('POSCAR', 'POSCAR.tmp')
                    vasp.regulate_poscar('POSCAR.tmp', 'POSCAR')
                    vasp.sort_poscar('POSCAR', 'POSCAR', ptypes)
                    vasp.perturb_xz('POSCAR', 'POSCAR', self.pert_xz)
                    # record miller
                    dumpfn(all_slabs[ii].miller_index, 'miller.json')
                os.chdir(cwd)

        return task_list

    def post_process(self, task_list):
        pass

    def task_type(self):
        return self.parameter['type']

    def task_param(self):
        return self.parameter

    def _compute_lower(self,
                       output_file,
                       all_tasks,
                       all_res):
        output_file = os.path.abspath(output_file)
        res_data = {}
        ptr_data = os.path.dirname(output_file) + '\n'

        if not self.reprod:
            ptr_data += "Miller_Indices: \tSurf_E(J/m^2) EpA(eV) equi_EpA(eV)\n"
            for ii in all_tasks:
                task_result = loadfn(os.path.join(ii, 'result_task.json'))
                natoms = np.sum(task_result['atom_numbs'])
                epa = task_result['energies'][-1] / natoms
                AA = np.linalg.norm(np.cross(task_result['cells'][0][0], task_result['cells'][0][1]))

                equi_path = os.path.abspath(os.path.join(os.path.dirname(output_file), '../relaxation/relax_task'))
                equi_result = loadfn(os.path.join(equi_path, 'result.json'))
                equi_epa = equi_result['energies'][-1] / np.sum(equi_result['atom_numbs'])
                structure_dir = os.path.basename(ii)

                Cf = 1.60217657e-16 / (1e-20 * 2) * 0.001
                evac = (task_result['energies'][-1] - equi_epa * natoms) / AA * Cf

                miller_index = loadfn(os.path.join(ii, 'miller.json'))
                ptr_data += "%-25s     %7.3f    %8.3f %8.3f\n" % (
                    str(miller_index) + '-' + structure_dir + ':', evac, epa, equi_epa)
                res_data[str(miller_index) + '-' + structure_dir] = [evac, epa, equi_epa]

        else:
            if 'init_data_path' not in self.parameter:
                raise RuntimeError("please provide the initial data path to reproduce")
            init_data_path = os.path.abspath(self.parameter['init_data_path'])
            res_data, ptr_data = post_repro(init_data_path, self.parameter['init_from_suffix'],
                                            all_tasks, ptr_data, self.parameter.get('reprod_last_frame', True))

        with open(output_file, 'w') as fp:
            json.dump(res_data, fp, indent=4)

        return res_data, ptr_data



import os,sys,json
import unittest

test_dir = os.path.abspath(os.path.join(os.path.dirname(__file__)))
sys.path.insert(0, os.path.join(test_dir, '..'))
__package__ = 'tools'
from dpgen.remote.decide_machine import convert_mdata
from .context import setUpModule

class TestConvertMdata(unittest.TestCase):
    machine_file = 'machine_fp_single.json'

    def test_convert_mdata (self):
        mdata = json.load(open(self.machine_file))
        mdata = convert_mdata(mdata, ["fp"])
        self.assertEqual(mdata["fp_command"], "vasp_std")
        self.assertEqual(mdata["fp_group_size"], 8)
        self.assertEqual(mdata["fp_machine"]["batch_type"], "PBS")
        self.assertEqual(mdata["fp_user_forward_files"], ["vdw_kernel.bindat"])


class TestConvertMdata2(TestConvertMdata):
    machine_file = 'machine_fp_single2.json'



import argparse
import copy
import dpdata
import glob
import math
import numpy as np
import os
import sys
import shutil
from deepmd.infer import calc_model_devi
from deepmd.infer import DeepPot as DP


def write_model_devi_out(devi, fname):
    assert devi.shape[1] == 8
    #assert devi.shape[1] == 7
    header = '%5s' % 'step'
    for item in 'vf':
        header += '%16s%16s%16s' % (f'max_devi_{item}', f'min_devi_{item}',f'avg_devi_{item}')
    header += '%16s'%str('min_dis')
    np.savetxt(fname,
               devi,
               fmt=['%5d'] + ['%17.6e' for _ in range(7)],
               delimiter='',
               header=header)
    return devi


def Modd(all_models,type_map):

    # Model Devi 

    cwd = os.getcwd()
    graphs = [DP(model) for model in all_models]

    Devis = []
    pcount = 0
    strus_lists = glob.glob(os.path.join(cwd,'*.structures'))
    for num, strus_path in enumerate(strus_lists):

        structures_data = dpdata.System(strus_path,'deepmd/npy',type_map=type_map)

        # every 500 confs in one task dir
        nnum =  structures_data.get_nframes()
        if nnum == 0:
            continue
        else:
            num_per_task = math.ceil(nnum/500)
            

        for temp in range(num_per_task):
            task_name = os.path.join(cwd,'task.%03d.%03d'%(num,temp)) 
            put_poscar = os.path.join(task_name,'traj')
            if not os.path.exists(task_name):
                os.mkdir(task_name)
                os.mkdir(put_poscar)
            else:
                shutil.rmtree(task_name)
                os.mkdir(task_name)
                os.mkdir(put_poscar)
            devis = []
            if (nnum - (temp+1)*500) >= 0:
                temp_sl = range(temp*500,(temp+1)*500)
            else:
                temp_sl = range(temp*500,nnum)
                
            new_index = 0
            for index,frameid in enumerate(temp_sl):
                pdata = structures_data[frameid]
                pdata.to_vasp_poscar(os.path.join(put_poscar,'%s.poscar'%str(index)))
                nopbc = pdata.nopbc
                coord = pdata.data['coords']
                cell  = pdata.data['cells']
                atom_types = pdata.data['atom_types']
                devi = calc_model_devi(coord,cell,atom_types,graphs,nopbc=nopbc)
                # ------------------------------------------------------------------------------------
                # append min-distance in devi list
                dis = pdata.to_ase_structure()[0].get_all_distances(mic=True)
                row,col = np.diag_indices_from(dis)
                dis[row,col] = 10000
                min_dis = np.nanmin(dis)
                devi = np.append(devi[0],min_dis)
                t = [devi]
                devi = np.array(t)
                # ------------------------------------------------------------------------------------
                temp_d = copy.deepcopy(devi)
                temp_D = copy.deepcopy(devi)
                devis.append(temp_d)
                Devis.append(temp_D)
                devis[index][0][0]  = np.array(index)
                Devis[pcount][0][0] = np.array(pcount)
                pcount += 1
                new_index += 1
            devis = np.vstack(devis)
            write_model_devi_out(devis,os.path.join(task_name, 'model_devi.out'))

    Devis = np.vstack(Devis)
    write_model_devi_out(Devis,os.path.join(cwd,'Model_Devi.out'))

    f = open(os.path.join(os.path.abspath(os.path.join(cwd,os.pardir)),'record.calypso'),'a+')
    f.write('4')
    f.close()

if __name__ == '__main__':

    cwd = os.getcwd()
    model_path = os.path.join(os.path.abspath(os.path.join(cwd,os.pardir)),'gen_stru_analy')
    parser = argparse.ArgumentParser(description='calc model-devi by `all_models` and `type_map`')
    parser.add_argument(
        '--all_models',
        type=str,
        nargs='+',
        default=model_path,
        help='the path of models which will be used to do model-deviation',
    )
    parser.add_argument(
        '--type_map',
        nargs='+',
        help='the type map of models which will be used to do model-deviation',
    )
    args = parser.parse_args()
    #print(vars(args))
    Modd(args.all_models,args.type_map)
    #Modd(sys.argv[1],sys.argv[2])



import os
import warnings
import dpgen.auto_test.lib.lammps as lammps
from dpgen import dlog
from monty.serialization import loadfn, dumpfn
from dpgen.auto_test.Task import Task
from dpgen.auto_test.lib.lammps import inter_deepmd, inter_meam, inter_eam_fs, inter_eam_alloy

supported_inter = ["deepmd", 'meam', 'eam_fs', 'eam_alloy']


class Lammps(Task):
    def __init__(self,
                 inter_parameter,
                 path_to_poscar):
        self.inter = inter_parameter
        self.inter_type = inter_parameter['type']
        self.type_map = inter_parameter['type_map']
        self.in_lammps = inter_parameter.get('in_lammps', 'auto')
        if self.inter_type == 'meam':
            self.model = list(map(os.path.abspath, inter_parameter['model']))
        else:
            self.model = os.path.abspath(inter_parameter['model'])
        self.path_to_poscar = path_to_poscar
        assert self.inter_type in supported_inter
        self.set_inter_type_func()

    def set_inter_type_func(self):

        if self.inter_type == "deepmd":
            self.inter_func = inter_deepmd

        elif self.inter_type == 'meam':
            self.inter_func = inter_meam

        elif self.inter_type == 'eam_fs':
            self.inter_func = inter_eam_fs

        else:
            self.inter_func = inter_eam_alloy

    def set_model_param(self):

        if self.inter_type == "deepmd":
            model_name = os.path.basename(self.model)
            deepmd_version = self.inter.get("deepmd_version", "1.2.0")
            self.model_param = {'model_name': [model_name],
                                'param_type': self.type_map,
                                'deepmd_version': deepmd_version}
        elif self.inter_type == 'meam':
            model_name = list(map(os.path.basename, self.model))
            self.model_param = {'model_name': [model_name],
                                'param_type': self.type_map}
        else:
            model_name = os.path.basename(self.model)
            self.model_param = {'model_name': [model_name],
                                'param_type': self.type_map}

    def make_potential_files(self,
                             output_dir):
        cwd = os.getcwd()
        if self.inter_type == 'meam':
            model_lib = os.path.basename(self.model[0])
            model_file = os.path.basename(self.model[1])
            os.chdir(os.path.join(output_dir, '../'))
            if os.path.islink(model_lib):
                link_lib = os.readlink(model_lib)
                if not os.path.abspath(link_lib) == self.model[0]:
                    os.remove(model_lib)
                    os.symlink(os.path.relpath(self.model[0]), model_lib)
            else:
                os.symlink(os.path.relpath(self.model[0]), model_lib)

            if os.path.islink(model_file):
                link_file = os.readlink(model_file)
                if not os.path.abspath(link_file) == self.model[1]:
                    os.remove(model_file)
                    os.symlink(os.path.relpath(self.model[1]), model_file)
            else:
                os.symlink(os.path.relpath(self.model[1]), model_file)
            os.chdir(output_dir)
            if not os.path.islink(model_lib):
                os.symlink(os.path.join('..', model_lib), model_lib)
            elif not os.path.join('..', model_lib) == os.readlink(model_lib):
                os.remove(model_lib)
                os.symlink(os.path.join('..', model_lib), model_lib)

            if not os.path.islink(model_file):
                os.symlink(os.path.join('..', model_file), model_file)
            elif not os.path.join('..', model_file) == os.readlink(model_file):
                os.remove(model_file)
                os.symlink(os.path.join('..', model_file), model_file)
            os.chdir(cwd)

        else:
            model_file = os.path.basename(self.model)
            os.chdir(os.path.join(output_dir, '../'))
            if os.path.islink(model_file):
                link_file = os.readlink(model_file)
                if not os.path.abspath(link_file) == self.model:
                    os.remove(model_file)
                    os.symlink(os.path.relpath(self.model), model_file)
            else:
                os.symlink(os.path.relpath(self.model), model_file)
            os.chdir(output_dir)
            if not os.path.islink(model_file):
                os.symlink(os.path.join('..', model_file), model_file)
            elif not os.path.join('..', model_file) == os.readlink(model_file):
                os.remove(model_file)
                os.symlink(os.path.join('..', model_file), model_file)
            os.chdir(cwd)

        dumpfn(self.inter, os.path.join(output_dir, 'inter.json'), indent=4)

    def make_input_file(self,
                        output_dir,
                        task_type,
                        task_param):
        lammps.cvt_lammps_conf(os.path.join(output_dir, 'POSCAR'), os.path.join(output_dir, 'conf.lmp'), lammps.element_list(self.type_map))

        # dumpfn(task_param, os.path.join(output_dir, 'task.json'), indent=4)

        etol = 1e-12
        ftol = 1e-6
        maxiter = 5000
        maxeval = 500000
        B0 = 70
        bp = 0
        ntypes = len(self.type_map)

        cal_type = task_param['cal_type']
        cal_setting = task_param['cal_setting']

        self.set_model_param()

        # deal with user input in.lammps for relaxation
        if os.path.isfile(self.in_lammps) and task_type == 'relaxation':
            with open(self.in_lammps, 'r') as fin:
                fc = fin.read()
        # user input in.lammps for property calculation
        if 'input_prop' in cal_setting and os.path.isfile(cal_setting['input_prop']):
            with open(os.path.abspath(cal_setting['input_prop']), 'r') as fin:
                fc = fin.read()

        else:
            if 'etol' in cal_setting:
                dlog.info("%s setting etol to %s" % (self.make_input_file.__name__, cal_setting['etol']))
                etol = cal_setting['etol']
            if 'ftol' in cal_setting:
                dlog.info("%s setting ftol to %s" % (self.make_input_file.__name__, cal_setting['ftol']))
                ftol = cal_setting['ftol']
            if 'maxiter' in cal_setting:
                dlog.info("%s setting maxiter to %s" % (self.make_input_file.__name__, cal_setting['maxiter']))
                maxiter = cal_setting['maxiter']
            if 'maxeval' in cal_setting:
                dlog.info("%s setting maxeval to %s" % (self.make_input_file.__name__, cal_setting['maxeval']))
                maxeval = cal_setting['maxeval']

            if cal_type == 'relaxation':
                relax_pos = cal_setting['relax_pos']
                relax_shape = cal_setting['relax_shape']
                relax_vol = cal_setting['relax_vol']

                if [relax_pos, relax_shape, relax_vol] == [True, False, False]:
                    fc = lammps.make_lammps_equi('conf.lmp', self.type_map, self.inter_func, self.model_param,
                                                 etol, ftol, maxiter, maxeval, False)
                elif [relax_pos, relax_shape, relax_vol] == [True, True, True]:
                    fc = lammps.make_lammps_equi('conf.lmp', self.type_map, self.inter_func, self.model_param,
                                                 etol, ftol, maxiter, maxeval, True)
                elif [relax_pos, relax_shape, relax_vol] == [True, True, False] and not task_type == 'eos':
                    if 'scale2equi' in task_param:
                        scale2equi = task_param['scale2equi']
                        fc = lammps.make_lammps_press_relax('conf.lmp', self.type_map, scale2equi[int(output_dir[-6:])],
                                                            self.inter_func,
                                                            self.model_param, B0, bp, etol, ftol, maxiter, maxeval)
                    else:
                        fc = lammps.make_lammps_equi('conf.lmp', self.type_map, self.inter_func, self.model_param,
                                                     etol, ftol, maxiter, maxeval, True)
                elif [relax_pos, relax_shape, relax_vol] == [True, True, False] and task_type == 'eos':
                    task_param['cal_setting']['relax_shape'] = False
                    fc = lammps.make_lammps_equi('conf.lmp', self.type_map, self.inter_func, self.model_param,
                                                 etol, ftol, maxiter, maxeval, False)
                elif [relax_pos, relax_shape, relax_vol] == [False, False, False]:
                    fc = lammps.make_lammps_eval('conf.lmp', self.type_map, self.inter_func, self.model_param)

                else:
                    raise RuntimeError("not supported calculation setting for LAMMPS")

            elif cal_type == 'static':
                fc = lammps.make_lammps_eval('conf.lmp', self.type_map, self.inter_func, self.model_param)

            else:
                raise RuntimeError("not supported calculation type for LAMMPS")

        dumpfn(task_param, os.path.join(output_dir, 'task.json'), indent=4)

        in_lammps_not_link_list = ['eos']
        if task_type not in in_lammps_not_link_list:
            with open(os.path.join(output_dir, '../in.lammps'), 'w') as fp:
                fp.write(fc)
            cwd = os.getcwd()
            os.chdir(output_dir)
            if not (os.path.islink('in.lammps') or os.path.isfile('in.lammps')):
                os.symlink('../in.lammps', 'in.lammps')
            else:
                os.remove('in.lammps')
                os.symlink('../in.lammps', 'in.lammps')
            os.chdir(cwd)
        else:
            with open(os.path.join(output_dir, 'in.lammps'), 'w') as fp:
                fp.write(fc)

    def compute(self,
                output_dir):
        log_lammps = os.path.join(output_dir, 'log.lammps')
        dump_lammps = os.path.join(output_dir, 'dump.relax')
        if not os.path.isfile(log_lammps):
            warnings.warn("cannot find log.lammps in " + output_dir + " skip")
            return None
        if not os.path.isfile(dump_lammps):
            warnings.warn("cannot find dump.relax in " + output_dir + " skip")
            return None
        else:
            box = []
            coord = []
            vol = []
            energy = []
            force = []
            virial = []
            stress = []
            with open(dump_lammps, 'r') as fin:
                dump = fin.read().split('\n')
            dumptime = []
            for idx, ii in enumerate(dump):
                if ii == 'ITEM: TIMESTEP':
                    box.append([])
                    coord.append([])
                    force.append([])
                    dumptime.append(int(dump[idx + 1]))
                    natom = int(dump[idx + 3])
                    xlo_bound = float(dump[idx + 5].split()[0])
                    xhi_bound = float(dump[idx + 5].split()[1])
                    xy = float(dump[idx + 5].split()[2])
                    ylo_bound = float(dump[idx + 6].split()[0])
                    yhi_bound = float(dump[idx + 6].split()[1])
                    xz = float(dump[idx + 6].split()[2])
                    zlo = float(dump[idx + 7].split()[0])
                    zhi = float(dump[idx + 7].split()[1])
                    yz = float(dump[idx + 7].split()[2])
                    xx = xhi_bound - max([0, xy, xz, xy + xz]) - (xlo_bound - min([0, xy, xz, xy + xz]))
                    yy = yhi_bound - max([0, yz]) - (ylo_bound - min([0, yz]))
                    zz = zhi - zlo
                    box[-1].append([xx, 0.0, 0.0])
                    box[-1].append([xy, yy, 0.0])
                    box[-1].append([xz, yz, zz])
                    vol.append(xx * yy * zz)
                    type_list = []
                    for jj in range(natom):
                        type_list.append(int(dump[idx + 9 + jj].split()[1]) - 1)
                        if 'xs ys zs' in dump[idx + 8]:
                            a_x = float(dump[idx + 9 + jj].split()[2]) * xx + float(
                                dump[idx + 9 + jj].split()[3]) * xy \
                                  + float(dump[idx + 9 + jj].split()[4]) * xz
                            a_y = float(dump[idx + 9 + jj].split()[3]) * yy + float(
                                dump[idx + 9 + jj].split()[4]) * yz
                            a_z = float(dump[idx + 9 + jj].split()[4]) * zz
                        else:
                            a_x = float(dump[idx + 9 + jj].split()[2])
                            a_y = float(dump[idx + 9 + jj].split()[3])
                            a_z = float(dump[idx + 9 + jj].split()[4])
                        coord[-1].append([a_x, a_y, a_z])
                        fx = float(dump[idx + 9 + jj].split()[5])
                        fy = float(dump[idx + 9 + jj].split()[6])
                        fz = float(dump[idx + 9 + jj].split()[7])
                        force[-1].append([fx, fy, fz])

            with open(log_lammps, 'r') as fp:
                if 'Total wall time:' not in fp.read():
                    warnings.warn("lammps not finished " + log_lammps + " skip")
                    return None
                else:
                    fp.seek(0)
                    lines = fp.read().split('\n')
                    idid = -1
                    for ii in dumptime:
                        idid += 1
                        for jj in lines:
                            line = jj.split()
                            if len(line) and str(ii) == line[0]:
                                try:
                                    [float(kk) for kk in line]
                                except Exception:
                                    continue
                                stress.append([])
                                virial.append([])
                                energy.append(float(line[1]))
                                # virials = stress * vol * 1e5 *1e-30 * 1e19/1.6021766208
                                stress[-1].append([float(line[2]) / 1000.0, float(line[5]) / 1000.0, float(line[6]) / 1000.0])
                                stress[-1].append([float(line[5]) / 1000.0, float(line[3]) / 1000.0, float(line[7]) / 1000.0])
                                stress[-1].append([float(line[6]) / 1000.0, float(line[7]) / 1000.0, float(line[4]) / 1000.0])
                                stress_to_virial = vol[idid] * 1e5 * 1e-30 * 1e19 / 1.6021766208
                                virial[-1].append([float(line[2]) * stress_to_virial, float(line[5]) * stress_to_virial,
                                                   float(line[6]) * stress_to_virial])
                                virial[-1].append([float(line[5]) * stress_to_virial, float(line[3]) * stress_to_virial,
                                                   float(line[7]) * stress_to_virial])
                                virial[-1].append([float(line[6]) * stress_to_virial, float(line[7]) * stress_to_virial,
                                                   float(line[4]) * stress_to_virial])
                                break

            _tmp = self.type_map
            dlog.debug(_tmp)
            type_map_list = lammps.element_list(self.type_map)

            type_map_idx = list(range(len(type_map_list)))
            atom_numbs = []
            for ii in type_map_idx:
                count = 0
                for jj in type_list:
                    if jj == ii:
                        count += 1
                atom_numbs.append(count)

            # d_dump = dpdata.System(dump_lammps, fmt='lammps/dump', type_map=type_map_list)
            # d_dump.to('vasp/poscar', contcar, frame_idx=-1)

            result_dict = {"@module": "dpdata.system", "@class": "LabeledSystem", "data": {"atom_numbs": atom_numbs,
                                                                                           "atom_names": type_map_list,
                                                                                           "atom_types": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "int64",
                                                                                               "data": type_list},
                                                                                           "orig": {"@module": "numpy",
                                                                                                    "@class": "array",
                                                                                                    "dtype": "int64",
                                                                                                    "data": [0, 0, 0]},
                                                                                           "cells": {"@module": "numpy",
                                                                                                     "@class": "array",
                                                                                                     "dtype": "float64",
                                                                                                     "data": box},
                                                                                           "coords": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "float64",
                                                                                               "data": coord},
                                                                                           "energies": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "float64",
                                                                                               "data": energy},
                                                                                           "forces": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "float64",
                                                                                               "data": force},
                                                                                           "virials": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "float64",
                                                                                               "data": virial},
                                                                                           "stress": {
                                                                                               "@module": "numpy",
                                                                                               "@class": "array",
                                                                                               "dtype": "float64",
                                                                                               "data": stress}}}

            contcar = os.path.join(output_dir, 'CONTCAR')
            dumpfn(result_dict, contcar, indent=4)
            d_dump = loadfn(contcar)
            d_dump.to('vasp/poscar', contcar, frame_idx=-1)

            return result_dict

    def forward_files(self, property_type='relaxation'):
        if self.inter_type == 'meam':
            return ['conf.lmp', 'in.lammps'] + list(map(os.path.basename, self.model))
        else:
            return ['conf.lmp', 'in.lammps', os.path.basename(self.model)]

    def forward_common_files(self, property_type='relaxation'):
        if property_type not in ['eos']:
            if self.inter_type == 'meam':
                return ['in.lammps'] + list(map(os.path.basename, self.model))
            else:
                return ['in.lammps', os.path.basename(self.model)]
        else:
            if self.inter_type == 'meam':
                return list(map(os.path.basename, self.model))
            else:
                return [os.path.basename(self.model)]

    def backward_files(self, property_type='relaxation'):
        return ['log.lammps', 'outlog', 'dump.relax']



import os, sys, json, glob, shutil
import dpdata
import numpy as np
import unittest
from dpdata import LabeledSystem
from pymatgen.io.vasp import Incar
from monty.serialization import loadfn, dumpfn

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'

from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.VASP import  VASP
from dpgen.generator.lib.vasp import incar_upper


class TestVASP(unittest.TestCase):

    def setUp(self):
        self.jdata = {
            "structures": ["confs/hp-*"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR",
                "potcar_prefix": ".",
                "potcars": {"Li": "vasp_input/POTCAR"}
            },
            "relaxation": {
                         "cal_type":     "relaxation",
                         "cal_setting":  {"relax_pos":True,
                                          "relax_shape":True,
                                          "relax_vol":True}
            }
        }

        self.conf_path = 'confs/hp-Li'
        self.equi_path = 'confs/hp-Li/relaxation/relax_task'
        self.source_path = 'equi/vasp'
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = self.jdata["structures"]
        inter_param = self.jdata["interaction"]
        self.task_param = self.jdata["relaxation"]
        self.VASP = VASP(inter_param, os.path.join(self.conf_path, 'POSCAR'))

    def tearDown(self):
        if os.path.exists('confs/hp-Li/relaxation'):
            shutil.rmtree('confs/hp-Li/relaxation')
        if os.path.exists('inter.json'):
            os.remove('inter.json')
        if os.path.exists('POTCAR'):
            os.remove('POTCAR')

    def test_make_potential_files(self):
        if not os.path.exists(os.path.join(self.equi_path, 'POSCAR')):
            with self.assertRaises(FileNotFoundError):
               self.VASP.make_potential_files(self.equi_path)
        shutil.copy(os.path.join(self.conf_path, 'POSCAR'), os.path.join(self.equi_path, 'POSCAR'))
        self.VASP.make_potential_files(self.equi_path)
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, "POTCAR")))
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, 'inter.json')))

    def test_make_input_file_1(self):
        param=self.task_param.copy()
        param["cal_setting"]= {"relax_pos":True,
                         "relax_shape":True,
                         "relax_vol":False}
        self.VASP.make_input_file(self.equi_path,'relaxation',param)
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],4)

    def test_make_input_file_2(self):
        self.VASP.make_input_file(self.equi_path,'relaxation',self.task_param)
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, "task.json")))   
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, "KPOINTS")))   
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, "INCAR")))   
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],3)

    def test_make_input_file_3(self):
        param=self.task_param.copy()
        param["cal_setting"]= {"relax_pos":True,
                         "relax_shape":False,
                         "relax_vol":False}
        self.VASP.make_input_file(self.equi_path,'relaxation',param)
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],2)

    def test_make_input_file_4(self):
        param=self.task_param.copy()
        param["cal_setting"]= {"relax_pos":False,
                         "relax_shape":True,
                         "relax_vol":False}
        self.VASP.make_input_file(self.equi_path,'relaxation',param)
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],5)

    def test_make_input_file_5(self):
        param=self.task_param.copy()
        param["cal_setting"]= {"relax_pos":False,
                         "relax_shape":True,
                         "relax_vol":True}
        self.VASP.make_input_file(self.equi_path,'relaxation',param)
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],6)

    def test_make_input_file_5(self):
        param=self.task_param.copy()
        param["cal_setting"]= {"relax_pos":False,
                         "relax_shape":True,
                         "relax_vol":True,
                         "kspacing":0.01}
        self.VASP.make_input_file(self.equi_path,'relaxation',param)
        incar=incar_upper(Incar.from_file(os.path.join(self.equi_path, "INCAR")))
        self.assertTrue(incar['ISIF'],6)
        self.assertTrue(incar['KSPACING'],0.01)

    def test_compuate(self):
        ret=self.VASP.compute(os.path.join(self.conf_path,'relaxation'))
        self.assertIsNone(ret)
        shutil.copy(os.path.join(self.source_path, 'OUTCAR'), os.path.join(self.equi_path, 'OUTCAR'))
        ret=self.VASP.compute(self.equi_path)
        ret_ref=loadfn(os.path.join(self.source_path, 'outcar.json'))
        self.assertTrue(ret,ret_ref.as_dict())
        
    def test_backward_files(self):
        backward_files = ['OUTCAR', 'outlog', 'CONTCAR', 'OSZICAR', 'XDATCAR']
        self.assertEqual(self.VASP.backward_files(), backward_files)



import os,sys,json,glob,shutil,textwrap
import dpdata
import numpy as np
import unittest
from pathlib import Path

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'

from .context import make_calypso_input
from .context import write_model_devi_out
from .context import _parse_calypso_input
from .context import _parse_calypso_dis_mtx

# temp dir
test_path = Path('.').joinpath('calypso_test_path')
test_path.mkdir(parents=True,exist_ok=True)
os.system('rm calypso_test_path/*')
fmax = 0.01
cwd = os.getcwd()

model_devi = np.array([[0.000000e+00, 2.328491e-02, 5.476687e-09, 1.009454e-02,
        3.279617e-02, 4.053224e-03, 1.869795e-02, 2.184905e+00],
       [1.000000e+00, 3.668334e-02, 8.200870e-09, 1.706517e-02,
        2.844074e-02, 7.093109e-03, 1.623275e-02, 2.424708e+00],
       [2.000000e+00, 2.832296e-02, 4.828951e-08, 1.573961e-02,
        2.443331e-02, 2.871548e-03, 1.489787e-02, 2.564113e+00]])

model_devi_jobs = {"model_devi_jobs": {"times":[4],"NameOfAtoms":["Mg","Al","Cu"],"NumberOfAtoms":[1,1,1],"NumberOfFormula":[1,4],"Volume":[30],"DistanceOfIon":[[1.48,1.44,1.59],[1.44,1.41,1.56],[1.59,1.56,1.70]],"PsoRatio":[0.6],"PopSize":[5],"MaxStep":[3],"ICode":[13],"Split":"T","VSC":"T","MaxNumAtom":[31],
  "CtrlRange":[[1,10],[1,10],[1,10]],"PSTRESS":[0],"fmax":[0.01]}}
    

class TestCALYPSOScript(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_write_model_devi_out(self):
        #devi = write_model_devi_out(model_devi, 'calypso_test_path/model_devi.out')
        #ndevi = np.loadtxt('calypso_test_path/model_devi.out')
        devi = write_model_devi_out(model_devi, 'model_devi.out')
        ndevi = np.loadtxt('model_devi.out')
        self.assertEqual(ndevi[2,4],model_devi[2,4])
        os.remove('model_devi.out')

    def test_make_calypso_input(self):
        ret = make_calypso_input(["Mg","Al","Cu"],[1,1,1],[1,4],[30],[
                              [1.48,1.44,1.59],[1.44,1.41,1.56],[1.59,1.56,1.70]
                              ],[0.6],[5],[3],[13],"T","T",[31],[[1,10],[1,10],[1,10]],[0],[0.01])
        #with open('calypso_test_path/input.dat','w') as fin:
        with open('input.dat','w') as fin:
            fin.write(ret)
        f = open('input.dat')
        #f = open('calypso_test_path/input.dat')
        lines = f.readlines()
        f.close()
        for line in lines :
            if line[0] == '#':
                continue
            if 'PopSize' in line:
                temp_1 = line.split('=')[1].strip()
                self.assertEqual(int(temp_1),5)
            if 'MaxStep' in line:
                temp_2 = line.split('=')[1].strip()
                self.assertEqual(int(temp_2),3)
                os.remove('input.dat')
                break

    def test_parse_calypso_input(self):
        ret = make_calypso_input(["Mg","Al","Cu"],[1,1,1],[1,4],[30],[
                              [1.48,1.44,1.59],[1.44,1.41,1.56],[1.59,1.56,1.70]
                              ],[0.6],[5],[3],[13],"T","T",[31],[[1,10],[1,10],[1,10]],[0],[0.01])
        #with open('calypso_test_path/input.dat','w') as fin:
        with open('input.dat','w') as fin:
            fin.write(ret)
        formula = _parse_calypso_input('NumberOfFormula','input.dat').split()
        #formula = _parse_calypso_input('NumberOfFormula',calypso_data).split()
        formula = list(map(int,formula))
        self.assertEqual(formula,model_devi_jobs.get('model_devi_jobs').get('NumberOfFormula'))

        nameofatoms = _parse_calypso_input('NameOfAtoms','input.dat').split()
        #nameofatoms = _parse_calypso_input('NameOfAtoms',calypso_data).split()
        self.assertEqual(nameofatoms,model_devi_jobs.get('model_devi_jobs').get('NameOfAtoms'))
        
        min_dis = _parse_calypso_dis_mtx(len(nameofatoms),'input.dat')
        #min_dis = _parse_calypso_dis_mtx(len(nameofatoms),calypso_data)
        self.assertEqual(float(min_dis),np.nanmin(model_devi_jobs.get('model_devi_jobs').get('DistanceOfIon')))
        os.remove('input.dat')


if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python3
import os
import shutil
import json
import numpy as np

def make_calypso_input(nameofatoms,numberofatoms,
                           numberofformula,volume,
                           distanceofion,psoratio,popsize,
                           maxstep,icode,split,vsc,
                           maxnumatom,ctrlrange,pstress,fmax):
    ret = "################################ The Basic Parameters of CALYPSO ################################\n"
    ret+= "# A string of one or several words contain a descriptive name of the system (max. 40 characters).\n"
    assert (nameofatoms != None )
    ret+= "SystemName = %s\n"%(''.join(nameofatoms))
    ret+= "# Number of different atomic species in the simulation.\n"
    ret+= "NumberOfSpecies = %d\n"%(len(nameofatoms))
    ret+= "# Element symbols of the different chemical species.\n"
    ret+= "NameOfAtoms = %s\n"%(' '.join(nameofatoms))
    ret+= "# Number of atoms for each chemical species in one formula unit. \n"
    assert numberofatoms != None  or len(numberofatoms) == len(nameofatoms)
    ret+= "NumberOfAtoms = %s\n"%(' '.join(list(map(str,numberofatoms))))
    ret+= "# The range of formula unit per cell in your simulation. \n"
    assert numberofformula != None  or len(numberofformula) == 2 or type(numberofformula) == type([1,2])
    ret+= "NumberOfFormula = %s\n"%(' '.join(list(map(str,numberofformula))))
    ret+= "# The volume per formula unit. Unit is in angstrom^3.\n"
    ret+= "Volume = %s\n"%(volume[0])
    ret+= "# Minimal distance between atoms of each chemical species. Unit is in angstrom.\n"
    assert len(distanceofion) == len(nameofatoms) #"check distance of ions and the number of atoms"
    assert len(distanceofion[0]) == len(nameofatoms)
    ret+= "@DistanceOfIon \n"
    for temp in distanceofion:
        ret+="%4s \n"%(' '.join(list(map(str,temp))))
    ret+= "@End\n"
    ret+= "# It determines which algorithm should be adopted in the simulation.\n"
    ret+= "Ialgo = 2\n"
    ret+= "# Ialgo = 1 for Global PSO\n"
    ret+= "# Ialgo = 2 for Local PSO (default value)\n"
    ret+= "# The proportion of the structures generated by PSO.\n"
    assert (0 <=  psoratio[0] <= 1  )
    ret+= "PsoRatio = %s\n"%(psoratio[0])
    ret+= "# The population size. Normally, it has a larger number for larger systems.\n"
    assert popsize[0] != None  or popsize!=None or type(popsize) == type([0,1]) or type(popsize[0]) == type(0)
    ret+= "PopSize = %d\n"%(popsize[0])
    assert maxstep[0] != None  or maxstep!=None or type(maxstep) == type([0,1]) or type(maxstep[0]) == type(0)
    ret+= "# The Max step for iteration\n"
    ret+= "MaxStep = %d\n"%(maxstep[0])
    ret+= "#It determines which method should be adopted in generation the random structure. \n"                             
    ret+= "GenType= 1 \n"
    ret+= "# 1 under symmetric constraints\n"
    ret+= "# 2 grid method for large system\n"
    ret+= "# 3 and 4 core grow method \n"
    ret+= "# 0 combination of all method\n"
    ret+= "# If GenType=3 or 4, it determined the small unit to grow the whole structure\n"
    ret+= "# It determines which local optimization method should be interfaced in the simulation.\n"
    assert icode != None  or type(icode) == type([0,1]) or type(icode[0]) == type(0)
    ret+= "ICode= %d\n"%(icode[0])
    ret+= "# ICode= 1 interfaced with VASP\n"
    ret+= "# ICode= 2 interfaced with SIESTA\n"
    ret+= "# ICode= 3 interfaced with GULP\n"
    ret+= "# The number of lbest for local PSO\n"
    ret+= "NumberOfLbest=4\n"
    ret+= "# The Number of local optimization for each structure.\n"
    ret+= "NumberOfLocalOptim= 3\n"
    ret+= "# The command to perform local optimiztion calculation (e.g., VASP, SIESTA) on your computer.\n"
    ret+= "Command = sh submit.sh\n"
    ret+= "MaxTime = 9000 \n"
    ret+= "# If True, a previous calculation will be continued.\n"
    ret+= "PickUp = F\n"
    ret+= "# At which step will the previous calculation be picked up.\n"
    ret+= "PickStep = 1\n"
    ret+= "# If True, the local optimizations performed by parallel\n"
    ret+= "Parallel = F\n"
    ret+= "# The number node for parallel \n"
    ret+= "NumberOfParallel = 4\n"
    assert split != None 
    ret+= "Split = %s\n"%(split)
    assert pstress != None  or type(pstress) == type([200])
    ret+= "PSTRESS = %f\n"%(pstress[0])
    assert fmax != None  or type(fmax) == type([200])
    ret+= "fmax = %f\n"%(fmax[0])
    ret+= "################################ End of The Basic Parameters of CALYPSO #######################\n"
    if vsc == 'T':
        assert len(ctrlrange) == len(nameofatoms) #'check distance of ions and the number of atoms'
        ret+= "##### The Parameters For Variational Stoichiometry  ##############\n"
        ret+= "## If True, Variational Stoichiometry structure prediction is performed\n"
        ret+= "VSC = %s\n"%(vsc)
        ret+= "# The Max Number of Atoms in unit cell\n"
        ret+= "MaxNumAtom = %s\n"%(maxnumatom[0])
        ret+= "# The Variation Range for each type atom \n"
        ret+= "@CtrlRange\n"                                                                                                      
        for ttemp in ctrlrange:
            ret+="%4s \n"%(' '.join(list(map(str,ttemp))))
        ret+= "@end\n"
        ret+= "###################End Parameters for VSC ##########################\n"
    return ret

def _make_model_devi_buffet(jdata,calypso_run_opt_path):

    calypso_input_path = jdata.get('calypso_input_path')
    shutil.copyfile(os.path.join(calypso_input_path,'input.dat'),os.path.join(calypso_run_opt_path, 'input.dat'))

def _make_model_devi_native_calypso(iter_index,model_devi_jobs, calypso_run_opt_path):

    for iiidx, jobbs in enumerate(model_devi_jobs):
        if iter_index in jobbs.get('times'):
            cur_job = model_devi_jobs[iiidx]

    work_path = os.path.dirname(calypso_run_opt_path)
    with open(os.path.join(work_path, 'cur_job.json'), 'w') as outfile:
        json.dump(cur_job, outfile, indent = 4)

    # Crystal Parameters
    nameofatoms = cur_job.get('NameOfAtoms')
    numberofatoms = cur_job.get('NumberOfAtoms')
    numberofformula = cur_job.get('NumberOfFormula',[1,1])
    volume = cur_job.get('Volume')
    distanceofion = cur_job.get('DistanceOfIon')
    psoratio = cur_job.get('PsoRatio')
    popsize = cur_job.get('PopSize')
    maxstep = cur_job.get('MaxStep')
    icode = cur_job.get('ICode',[1])
    split = cur_job.get('Split','T')
    # VSC Control
    maxnumatom = None
    ctrlrange = None
    vsc = cur_job.get('VSC','F')
    if vsc == 'T':
        maxnumatom = cur_job.get('MaxNumAtom')
        ctrlrange = cur_job.get('CtrlRange')

    # Optimization
    pstress = cur_job.get('PSTRESS',[0.001])
    fmax = cur_job.get('fmax',[0.01])

    # Cluster

    # 2D

    file_c = make_calypso_input(nameofatoms,numberofatoms,
                               numberofformula,volume,
                               distanceofion,psoratio,popsize,
                               maxstep,icode,split,vsc,
                               maxnumatom,ctrlrange,pstress,fmax)
    with open(os.path.join(calypso_run_opt_path, 'input.dat'), 'w') as cin :
        cin.write(file_c)

def write_model_devi_out(devi, fname):
    assert devi.shape[1] == 8
    #assert devi.shape[1] == 7
    header = '%5s' % 'step'
    for item in 'vf':
        header += '%16s%16s%16s' % (f'max_devi_{item}', f'min_devi_{item}',f'avg_devi_{item}')
    header += '%16s'%str('min_dis')
    np.savetxt(fname,
               devi,
               fmt=['%5d'] + ['%17.6e' for _ in range(7)],
               delimiter='',
               header=header)
    return devi




import sys,os
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))
from dpgen.data.gen import *

param_file = 'al.json'
abacus_param_file = 'ch4.json'
def setUpModule():
    os.chdir(os.path.abspath(os.path.dirname(__file__)))



import os,sys,json,glob,shutil,textwrap
import dpdata
import numpy as np
import unittest

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'generator'
from .context import get_dumped_forces

class TestGetDumpForce(unittest.TestCase):
    def setUp(self):
        file_content = textwrap.dedent("""\
ITEM: TIMESTEP
40
ITEM: NUMBER OF ATOMS
2
ITEM: BOX BOUNDS xy xz yz pp pp pp
-2.9180686220264818e-04 8.0855380329747089e+00 1.4011011277606830e-07
-2.9198257591541018e-04 8.0855378881632269e+00 3.3202396460852749e-08
-2.9180686326490957e-04 8.0855378891632768e+00 -1.7571268247505500e-07
ITEM: ATOMS id type x y z fx fy fz
1 1 2.09532 8.19528 2.00538 -0.00569269 -0.0200373 -0.0342394
2 1 -0.0727384 4.01773 4.05582 -0.0297083 0.0817184 0.0722508
""")
        with open('tmp.dump', 'w') as fp:
            fp.write(file_content)
        self.expected_f = [ -0.00569269, -0.0200373, -0.0342394, -0.0297083, 0.0817184, 0.0722508]

    def tearDown(self):
        if os.path.isfile('tmp.dump'):
            os.remove('tmp.dump')

    def test_read_dump(self):
        ff = get_dumped_forces('tmp.dump')
        self.assertEqual(ff.shape, (2, 3))
        ff = ff.reshape([-1])
        for ii in range(6):
            self.assertAlmostEqual(ff[ii], self.expected_f[ii])



import os, sys, json, glob, shutil
import dpdata
import numpy as np
import unittest
from monty.serialization import loadfn, dumpfn

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'

from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.lib.lammps import inter_deepmd
from dpgen.auto_test.Lammps import Lammps


class TestLammps(unittest.TestCase):

    def setUp(self):
        self.jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "deepmd",
                "model": "lammps_input/frozen_model.pb",
                "deepmd_version": "1.1.0",
                "type_map": {"Al": 0}
            },
            "relaxation": {
                "cal_type": "relaxation",
                "cal_setting": {"relax_pos": True,
                                "relax_shape": True,
                                "relax_vol": True}
            }
        }

        self.equi_path = 'confs/std-fcc/relaxation/relax_task'
        self.source_path = 'equi/lammps'

        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        if not os.path.isfile(os.path.join(self.equi_path,'POSCAR')):
           shutil.copy(os.path.join(self.source_path, 'Al-fcc.vasp'), os.path.join('confs/std-fcc','POSCAR'))
             

        self.confs = self.jdata["structures"]
        self.inter_param = self.jdata["interaction"]
        self.relax_param = self.jdata["relaxation"]
        self.Lammps = Lammps(self.inter_param, os.path.join(self.source_path, 'Al-fcc.vasp'))

    def tearDown(self):
        if os.path.exists('confs/std-fcc/relaxation'):
            shutil.rmtree('confs/std-fcc/relaxation')

    def test_set_inter_type_func(self):
        self.Lammps.set_inter_type_func()
        self.assertEqual(inter_deepmd, self.Lammps.inter_func)

    def test_set_model_param(self):
        self.Lammps.set_model_param()
        model_param = {'model_name': ['frozen_model.pb'],
                       'param_type': {"Al": 0},
                       'deepmd_version': '1.1.0'}
        self.assertEqual(model_param, self.Lammps.model_param)

    def test_make_potential_files(self):
        cwd=os.getcwd()
        abs_equi_path=os.path.abspath(self.equi_path)
        self.Lammps.make_potential_files(abs_equi_path)
        self.assertTrue(os.path.islink(os.path.join(self.equi_path, "frozen_model.pb")))
        self.assertTrue(os.path.isfile(os.path.join(self.equi_path, 'inter.json')))
        ret=loadfn(os.path.join(self.equi_path, 'inter.json'))
        self.assertTrue(self.inter_param,ret)
        os.chdir(cwd)

    def test_make_input_file(self):
        cwd=os.getcwd()
        abs_equi_path=os.path.abspath('confs/std-fcc/relaxation/relax_task')
        shutil.copy(os.path.join('confs/std-fcc','POSCAR'), os.path.join(self.equi_path, 'POSCAR'))
        self.Lammps.make_input_file(abs_equi_path,'relaxation', self.relax_param)
        self.assertTrue(os.path.isfile(os.path.join(abs_equi_path, "conf.lmp")))
        self.assertTrue(os.path.islink(os.path.join(abs_equi_path, "in.lammps")))
        self.assertTrue(os.path.isfile(os.path.join(abs_equi_path, "task.json")))

    def test_forward_common_files(self):
        fc_files = ['in.lammps', 'frozen_model.pb']
        self.assertEqual(self.Lammps.forward_common_files(), fc_files)

    def test_backward_files(self):
        backward_files = ['log.lammps', 'outlog', 'dump.relax']
        self.assertEqual(self.Lammps.backward_files(), backward_files)



import glob
import os
from shutil import copyfile
import re

from monty.serialization import loadfn, dumpfn
from pymatgen.analysis.elasticity.elastic import ElasticTensor
from pymatgen.analysis.elasticity.strain import DeformedStructureSet, Strain
from pymatgen.analysis.elasticity.stress import Stress
from pymatgen.core.structure import Structure
from pymatgen.io.vasp import Incar, Kpoints

import dpgen.auto_test.lib.vasp as vasp
from dpgen import dlog
from dpgen.auto_test.Property import Property
from dpgen.auto_test.refine import make_refine
from dpgen.generator.lib.vasp import incar_upper


class Elastic(Property):
    def __init__(self,
                 parameter):
        if not ('init_from_suffix' in parameter and 'output_suffix' in parameter):
            default_norm_def = 2e-3
            default_shear_def = 5e-3
            parameter['norm_deform'] = parameter.get('norm_deform', default_norm_def)
            self.norm_deform = parameter['norm_deform']
            parameter['shear_deform'] = parameter.get('shear_deform', default_shear_def)
            self.shear_deform = parameter['shear_deform']
        parameter['cal_type'] = parameter.get('cal_type', 'relaxation')
        self.cal_type = parameter['cal_type']
        default_cal_setting = {"relax_pos": True,
                               "relax_shape": False,
                               "relax_vol": False}
        if 'cal_setting' not in parameter:
            parameter['cal_setting'] = default_cal_setting
        else:
            if "relax_pos" not in parameter['cal_setting']:
                parameter['cal_setting']['relax_pos'] = default_cal_setting['relax_pos']
            if "relax_shape" not in parameter['cal_setting']:
                parameter['cal_setting']['relax_shape'] = default_cal_setting['relax_shape']
            if "relax_vol" not in parameter['cal_setting']:
                parameter['cal_setting']['relax_vol'] = default_cal_setting['relax_vol']
        self.cal_setting = parameter['cal_setting']
        # parameter['reproduce'] = False
        # self.reprod = parameter['reproduce']
        self.parameter = parameter

    def make_confs(self,
                   path_to_work,
                   path_to_equi,
                   refine=False):
        path_to_work = os.path.abspath(path_to_work)
        if os.path.exists(path_to_work):
            dlog.warning('%s already exists' % path_to_work)
        else:
            os.makedirs(path_to_work)
        path_to_equi = os.path.abspath(path_to_equi)

        if 'start_confs_path' in self.parameter and os.path.exists(self.parameter['start_confs_path']):
            init_path_list = glob.glob(os.path.join(self.parameter['start_confs_path'], '*'))
            struct_init_name_list = []
            for ii in init_path_list:
                struct_init_name_list.append(ii.split('/')[-1])
            struct_output_name = path_to_work.split('/')[-2]
            assert struct_output_name in struct_init_name_list
            path_to_equi = os.path.abspath(os.path.join(self.parameter['start_confs_path'],
                                                        struct_output_name, 'relaxation', 'relax_task'))

        task_list = []
        cwd = os.getcwd()
        equi_contcar = os.path.join(path_to_equi, 'CONTCAR')

        os.chdir(path_to_work)
        if os.path.isfile('POSCAR'):
            os.remove('POSCAR')
        if os.path.islink('POSCAR'):
            os.remove('POSCAR')
        os.symlink(os.path.relpath(equi_contcar), 'POSCAR')
        #           task_poscar = os.path.join(output, 'POSCAR')

        # stress, deal with unsupported stress in dpdata
        #with open(os.path.join(path_to_equi, 'result.json')) as fin:
        #    equi_result = json.load(fin)
        #equi_stress = np.array(equi_result['stress']['data'])[-1]
        equi_result = loadfn(os.path.join(path_to_equi, 'result.json'))
        equi_stress = equi_result['stress'][-1]
        dumpfn(equi_stress, 'equi.stress.json', indent=4)
        os.chdir(cwd)

        if refine:
            print('elastic refine starts')
            task_list = make_refine(self.parameter['init_from_suffix'],
                                    self.parameter['output_suffix'],
                                    path_to_work)

            # record strain
            # df = Strain.from_deformation(dfm_ss.deformations[idid])
            # dumpfn(df.as_dict(), 'strain.json', indent=4)
            init_from_path = re.sub(self.parameter['output_suffix'][::-1],
                                    self.parameter['init_from_suffix'][::-1],
                                    path_to_work[::-1], count=1)[::-1]
            task_list_basename = list(map(os.path.basename, task_list))

            for ii in task_list_basename:
                init_from_task = os.path.join(init_from_path, ii)
                output_task = os.path.join(path_to_work, ii)
                os.chdir(output_task)
                if os.path.isfile('strain.json'):
                    os.remove('strain.json')
                copyfile(os.path.join(init_from_task, 'strain.json'), 'strain.json')
                #os.symlink(os.path.relpath(
                #    os.path.join((re.sub(self.parameter['output_suffix'], self.parameter['init_from_suffix'], ii)),
                #                 'strain.json')),
                #           'strain.json')
            os.chdir(cwd)
        else:
            norm_def = self.norm_deform
            shear_def = self.shear_deform
            norm_strains = [-norm_def, -0.5 * norm_def, 0.5 * norm_def, norm_def]
            shear_strains = [-shear_def, -0.5 * shear_def, 0.5 * shear_def, shear_def]

            if not os.path.exists(equi_contcar):
                raise RuntimeError("please do relaxation first")

            ss = Structure.from_file(equi_contcar)
            dfm_ss = DeformedStructureSet(ss,
                                          symmetry=False,
                                          norm_strains=norm_strains,
                                          shear_strains=shear_strains)
            n_dfm = len(dfm_ss)

            print('gen with norm ' + str(norm_strains))
            print('gen with shear ' + str(shear_strains))
            for ii in range(n_dfm):
                output_task = os.path.join(path_to_work, 'task.%06d' % ii)
                os.makedirs(output_task, exist_ok=True)
                os.chdir(output_task)
                for jj in ['INCAR', 'POTCAR', 'POSCAR', 'conf.lmp', 'in.lammps']:
                    if os.path.exists(jj):
                        os.remove(jj)
                task_list.append(output_task)
                dfm_ss.deformed_structures[ii].to('POSCAR', 'POSCAR')
                # record strain
                df = Strain.from_deformation(dfm_ss.deformations[ii])
                dumpfn(df.as_dict(), 'strain.json', indent=4)
            os.chdir(cwd)
        return task_list

    def post_process(self, task_list):
        cwd = os.getcwd()
        poscar_start = os.path.abspath(os.path.join(task_list[0], '..', 'POSCAR'))
        os.chdir(os.path.join(task_list[0], '..'))
        if os.path.isfile(os.path.join(task_list[0], 'INCAR')):
            incar = incar_upper(Incar.from_file(os.path.join(task_list[0], 'INCAR')))
            kspacing = incar.get('KSPACING')
            kgamma = incar.get('KGAMMA', False)
            ret = vasp.make_kspacing_kpoints(poscar_start, kspacing, kgamma)
            kp = Kpoints.from_string(ret)
            if os.path.isfile('KPOINTS'):
                os.remove('KPOINTS')
            kp.write_file("KPOINTS")
            os.chdir(cwd)
            kpoints_universal = os.path.abspath(os.path.join(task_list[0], '..', 'KPOINTS'))
            for ii in task_list:
                if os.path.isfile(os.path.join(ii, 'KPOINTS')):
                    os.remove(os.path.join(ii, 'KPOINTS'))
                if os.path.islink(os.path.join(ii, 'KPOINTS')):
                    os.remove(os.path.join(ii, 'KPOINTS'))
                os.chdir(ii)
                os.symlink(os.path.relpath(kpoints_universal), 'KPOINTS')
        os.chdir(cwd)

    def task_type(self):
        return self.parameter['type']

    def task_param(self):
        return self.parameter

    def _compute_lower(self,
                       output_file,
                       all_tasks,
                       all_res):
        output_file = os.path.abspath(output_file)
        res_data = {}
        ptr_data = os.path.dirname(output_file) + '\n'
        equi_stress = Stress(loadfn(os.path.join(os.path.dirname(output_file), 'equi.stress.json')))
        lst_strain = []
        lst_stress = []
        for ii in all_tasks:
            strain = loadfn(os.path.join(ii, 'strain.json'))
            # stress, deal with unsupported stress in dpdata
            #with open(os.path.join(ii, 'result_task.json')) as fin:
            #    task_result = json.load(fin)
            #stress = np.array(task_result['stress']['data'])[-1]
            stress = loadfn(os.path.join(ii, 'result_task.json'))['stress'][-1]
            lst_strain.append(strain)
            lst_stress.append(Stress(stress * -1000))

        et = ElasticTensor.from_independent_strains(lst_strain, lst_stress, eq_stress=equi_stress, vasp=False)
        res_data['elastic_tensor'] = []
        for ii in range(6):
            for jj in range(6):
                res_data['elastic_tensor'].append(et.voigt[ii][jj] / 1e4)
                ptr_data += "%7.2f " % (et.voigt[ii][jj] / 1e4)
            ptr_data += '\n'

        BV = et.k_voigt / 1e4
        GV = et.g_voigt / 1e4
        EV = 9 * BV * GV / (3 * BV + GV)
        uV = 0.5 * (3 * BV - 2 * GV) / (3 * BV + GV)

        res_data['BV'] = BV
        res_data['GV'] = GV
        res_data['EV'] = EV
        res_data['uV'] = uV
        ptr_data += "# Bulk   Modulus BV = %.2f GPa\n" % BV
        ptr_data += "# Shear  Modulus GV = %.2f GPa\n" % GV
        ptr_data += "# Youngs Modulus EV = %.2f GPa\n" % EV
        ptr_data += "# Poission Ratio uV = %.2f\n " % uV

        dumpfn(res_data, output_file, indent=4)

        return res_data, ptr_data



import os, sys, json, glob, shutil
import dpdata
import numpy as np
import unittest
import dpdata
from monty.serialization import loadfn, dumpfn
from pymatgen.analysis.elasticity.strain import Strain, Deformation
from pymatgen.core import Structure
from pymatgen.io.vasp import Incar

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'

from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.Elastic import Elastic


class TestElastic(unittest.TestCase):

    def setUp(self):
        _jdata = {
            "structures": ["confs/std-fcc"],
            "interaction": {
                "type": "vasp",
                "incar": "vasp_input/INCAR.rlx",
                "potcar_prefix": ".",
                "potcars": {"Al": "vasp_input/POT_Al"}
            },
            "properties": [
                {
                    "skip":False,
                    "type": "elastic",
                    "norm_deform": 2e-2,
                    "shear_deform": 5e-2
                }
            ]
        }

        self.equi_path = 'confs/std-fcc/relaxation/task_relax'
        self.source_path = 'equi/vasp'
        self.target_path = 'confs/std-fcc/elastic_00'
        if not os.path.exists(self.equi_path):
            os.makedirs(self.equi_path)

        self.confs = _jdata["structures"]
        self.inter_param = _jdata["interaction"]
        self.prop_param = _jdata['properties']

        self.elastic = Elastic(_jdata['properties'][0])

    def tearDown(self):
        if os.path.exists(os.path.join(self.equi_path,'..')):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.equi_path):
            shutil.rmtree(self.equi_path)
        if os.path.exists(self.target_path):
            shutil.rmtree(self.target_path)

    def test_task_type(self):
        self.assertEqual('elastic', self.elastic.task_type())

    def test_task_param(self):
        self.assertEqual(self.prop_param[0], self.elastic.task_param())

    def test_make_confs(self):

        shutil.copy(os.path.join(self.source_path, 'Al-fcc.json'), os.path.join(self.equi_path, 'result.json'))
        if not os.path.exists(os.path.join(self.equi_path, 'CONTCAR')):
            with self.assertRaises(RuntimeError):
                self.elastic.make_confs(self.target_path, self.equi_path)
        shutil.copy(os.path.join(self.source_path, 'CONTCAR_Al_fcc'), os.path.join(self.equi_path, 'CONTCAR'))
        task_list = self.elastic.make_confs(self.target_path, self.equi_path)
        dfm_dirs = glob.glob(os.path.join(self.target_path, 'task.*'))

        incar0 = Incar.from_file(os.path.join('vasp_input', 'INCAR.rlx'))
        incar0['ISIF'] = 4

        self.assertEqual(os.path.realpath(os.path.join(self.equi_path, 'CONTCAR')),
                         os.path.realpath(os.path.join(self.target_path, 'POSCAR')))
        ref_st = Structure.from_file(os.path.join(self.target_path, 'POSCAR'))
        dfm_dirs.sort()
        for ii in dfm_dirs:
            st_file = os.path.join(ii, 'POSCAR')
            self.assertTrue(os.path.isfile(st_file))
            strain_json_file = os.path.join(ii, 'strain.json')
            self.assertTrue(os.path.isfile(strain_json_file))



import os, sys, json, glob, shutil
import dpdata
import numpy as np
import unittest
from monty.serialization import loadfn, dumpfn
from pymatgen.io.vasp import Incar

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
__package__ = 'auto_test'

from .context import make_kspacing_kpoints
from .context import setUpModule

from dpgen.auto_test.common_prop import make_property
from dpgen.auto_test.refine import make_refine


class TestMakeProperty(unittest.TestCase):
    jdata = {
        "structures": ["confs/std-fcc"],
        "interaction": {
            "type": "vasp",
            "incar": "vasp_input/INCAR.rlx",
            "potcar_prefix": "vasp_input",
            "potcars": {"Al": "POT_Al"}
        },
        "properties": [
                    {
                      "type":         "eos",
                      "skip":  False,
                      "vol_start":    0.8,
                      "vol_end":      1.2,
                      "vol_step":     0.01,
                      "cal_setting": {
                                      "relax_pos": True,
                                      "relax_shape": True,
                                      "relax_vol": False,
                                      "overwrite_interaction":{
                                                  "type": "vasp",
                                                  "incar": "vasp_input/INCAR.rlx",
                                                  "potcar_prefix":"vasp_input",
                                                  "potcars": {"Al": "POT_Al"}
                                                  }
                                     }
                     }
        ]
    }

    def tearDown(self):
        if os.path.exists('confs/std-fcc/eos_00'):
            shutil.rmtree('confs/std-fcc/eos_00')
        if os.path.exists('confs/std-fcc/eos_02'):
            shutil.rmtree('confs/std-fcc/eos_02')
        if os.path.exists('confs/std-fcc/relaxation'):
            shutil.rmtree('confs/std-fcc/relaxation')

    def test_make_eos(self):

        pwd=os.getcwd()
        confs = self.jdata["structures"]
        inter_param = self.jdata["interaction"]
        property_list = self.jdata["properties"]

        target_path_0 = 'confs/std-fcc/eos_00'
        target_path_2 = 'confs/std-fcc/eos_02'
        equi_path = 'confs/std-fcc/relaxation/relax_task'
        source_path = 'equi/vasp'

        if not os.path.exists(equi_path):
            os.makedirs(equi_path)
        shutil.copy(os.path.join(source_path, 'CONTCAR_Al_fcc'), os.path.join(equi_path, 'CONTCAR'))

        make_property(confs, inter_param, property_list)

        dfm_dirs_0 = glob.glob(os.path.join(target_path_0, 'task.*'))

        for ii in dfm_dirs_0:
            self.assertTrue(os.path.isfile(os.path.join(ii, 'POSCAR')))
            shutil.copy(os.path.join(ii, 'POSCAR'),os.path.join(ii, 'CONTCAR'))

        path_to_work = os.path.abspath(target_path_0)
        new_prop_list=[
        {
         "type":         "eos",
         "init_from_suffix": "00",
         "output_suffix": "02",
         "cal_setting": {
                           "relax_pos": True,
                            "relax_shape": True,
                            "relax_vol": False,
         "input_prop":  "lammps_input/lammps_high"}
        }
        ]
        #ret=make_refine('00', '02', path_to_work)
        #self.assertEqual(len(ret),len(dfm_dirs_0))
        make_property(confs, inter_param, new_prop_list)
        self.assertTrue(os.path.isdir(path_to_work.replace('00','02')))
        os.chdir(pwd)
        dfm_dirs_2 = glob.glob(os.path.join(target_path_2, 'task.*'))
        self.assertEqual(len(dfm_dirs_2),len(dfm_dirs_0))
